# coding=utf-8
'''
xiami music provider.
'''
import re
import json
import logging
import urllib.parse

from apiRequestsBase import HttpRequest, ignored


logger = logging.getLogger(__name__)


# https://github.com/Flowerowl/xiami
def caesar(location):
    num = int(location[0])
    avg_len = int(len(location[1:]) / num)
    remainder = int(len(location[1:]) % num)
    result = [
        location[i * (avg_len + 1) + 1: (i + 1) * (avg_len + 1) + 1]
        for i in range(remainder)]
    result.extend(
        [
            location[(avg_len + 1) * remainder:]
            [i * avg_len + 1: (i + 1) * avg_len + 1]
            for i in range(num - remainder)])
    url = urllib.unquote(
        ''.join([
            ''.join([result[j][i] for j in range(num)])
            for i in range(avg_len)
        ]) +
        ''.join([result[r][-1] for r in range(remainder)])).replace('^', '0')
    return url


class XiamiApi(HttpRequest):

    default_timeout = 3.05

    def __init__(self):
        super(XiamiApi, self).__init__()
        self.lyricHeader = self.headers.copy()

        self.headers['Host'] = 'api.xiami.com'
        self.headers['Referer'] = 'http://m.xiami.com/'

        self.lyricHeader['Host'] = 'img.xiami.net'
        self.lyricHeader['Referer'] = 'http://img.xiami.net'

    def httpRequest(self, *args, **kwargs):
        html = super(XiamiApi, self).httpRequest(*args, **kwargs)

        logger.info("进行xiami Url请求, args: {0}, kwargs: {1}".format(args, kwargs))
       
        with ignored():
            return html.text
        
        logger.info("url: {0} 请求失败. Header: {1}".format(args[0], kwargs.get('headers')))
        return ''

    def playList(self, page=1):
        url = 'http://api.xiami.com/web?v=2.0&app_key=1&_ksTS=1459927525542_91' + \
            '&page={0}&limit=30&callback=jsonp92&r=collect/recommend'.format(page)
        
        response = self.httpRequest(url, method='GET')

        with ignored():
            response = json.loads(response[len('jsonp92('):-len(')')])
            return response['data']
        
        return False

    def getPlaylist(self, ids):
        url = 'http://api.xiami.com/web?v=2.0&app_key=1&id={0}'.format(ids) + \
        '&_ksTS=1459928471147_121&callback=jsonp122&r=collect/detail'
        response = self.httpRequest(url, 'GET')

        with ignored():
            response = json.loads(response[len('jsonp122('):-len(')')])
            # 修改下返回的信息，与之前的网易云对应，这样就不用修改太多即可做扩展。
            data = response['data']
            data['name'] = data['collect_name']
            data['creator'] = {'nickname': data['user_name']}
            data['description'] = ''
            data['trackCount'] = data['songs_count']
            songs = data['songs']
            for i in songs:
                i['name'] = i['song_name']
                i['artists'] = [{'name':i['singers']}]
                i['duration'] = int(i['length'])*1000
                i['album'] = {'blurPicUrl': i['album_logo']}

            data['tracks'] = songs
            return response['data']

        return False

    def search(self, keyword):
        keyword = urllib.parse.quote(keyword.encode("utf-8"))
        url = 'http://api.xiami.com/web?v=2.0&app_key=1&key=' + keyword \
        + '&page=1&limit=50&_ksTS=1459930568781_153&callback=jsonp154' + \
        '&r=search/songs'
        response = self.httpRequest(url, method='GET')
        response = json.loads(response[len('jsonp154('):-len(')')])
        # songs = response['data']['songs']
        response['data']['songCount'] = 50
        temp = []
        for songs in response['data']['songs']:
            temp.append({'name': songs['song_name'],
                                          'ar': [{'name': songs['artist_name']}],
                                          'al': {'picUrl': songs['album_logo']},
                                          # 虾米音乐搜索没有返回歌曲长度。
                                          'dt': 131400,
                                          'id': str(songs['song_id']),
                                          'mp3Url': songs['listen_file'],
                                          'lyric': songs['lyric']})

        response['data']['songs'] = temp

        return response['data']

    def lyric(self, url):
        lyric = self.httpRequest(url, method='GET', headers=self.lyricHeader)

        with ignored():
            lyric = lyric.split('[offset:0]')[1]
            lyric = re.sub(r'<\d*?>', '', lyric)
            return lyric

        with ignored():
            loadingLyric = {}
            lyricTimes = []
            for i in lyric.split('\n'):
                oneLyric = re.findall(r'[0-9:\.]+', i)
                if not oneLyric or oneLyric == [':']:
                    continue
                else:
                    for x in oneLyric:
                        i = i.replace('[{}]'.format(x), '')
                    for x in oneLyric:
                        loadingLyric[x] = i
                    lyricTimes.extend(oneLyric)

            return '\n'.join(['[{time}]{content}'.format(time=x, content=loadingLyric.get(x)) for x in sorted(lyricTimes)])

        return False


xiami = XiamiApi()


if __name__ == '__main__':
    a = xiami.get_url_by_id('326900802')

# def get_url_by_id(song_id):
#     url = 'http://www.xiami.com/song/playlist/id/%s' % song_id + \
#         '/object_name/default/object_id/0/cat/json'
#     response = h(url)
#     secret = json.loads(response)['data']['trackList'][0]['location']
#     url = caesar(secret)
#     return url


__author__ = 'cyrbuzz'

import addition

from base import QObject, QTableWidgetItem

transTime = addition.itv2time


class ConfigRecommendFrame(QObject):

    def __init__(self, parent):
        super().__init__()

        self.recommendFrame = parent

        self.musicList = []

        self.bindConnect()

    def setSongs(self, musicInfo):
        self.recommendFrame.singsTable.setRowCount(len(musicInfo))

        for index, data in enumerate(musicInfo):
            self.musicList.append(data._asdict())
            self.recommendFrame.singsTable.setItem(index, 0, QTableWidgetItem(data.name))
            self.recommendFrame.singsTable.setItem(index, 1, QTableWidgetItem(data.author))
            self.recommendFrame.singsTable.setItem(index, 2, QTableWidgetItem(transTime(data.time/1000)))

    def bindConnect(self):
        self.recommendFrame.singsTable.itemDoubleClicked.connect(self.itemDoubleClickedEvent)

    def itemDoubleClickedEvent(self):
        currentRow = self.recommendFrame.singsTable.currentRow()
        data = self.musicList[currentRow]

        self.recommendFrame.parent.playWidgets.setPlayerAndPlayList(data)



__author__ = 'cyrbuzz'
# 方便扩展，抽象成基类。
from base import (QCursor, QFrame, Qt, QTabWidget, QTextEdit,  QLabel, QIcon, QPushButton, QHBoxLayout, QVBoxLayout, 
                                     QGridLayout, QTableWidgetItem, PicLabel, ScrollArea, TableWidget, VBoxLayout, HBoxLayout, pyqtSignal)

import addition


class SingsFrameBase(ScrollArea):
    """全部歌单。"""

    def __init__(self, parent=None):
        super(SingsFrameBase, self).__init__()
        self.parent = parent
        self.transTime = addition.itv2time

        self.setObjectName("allSingsArea")
        # 为什么有的需要加utf-8呢，因为有中文。
        with open('QSS/singsFrameBase.qss', 'r', encoding='utf-8') as f:
            self.setStyleSheet(f.read())
        
        # 主布局。
        self.mainLayout = QGridLayout(self.frame)
        self.mainLayout.setSpacing(0)
        self.mainLayout.setHorizontalSpacing(10)
        self.mainLayout.setContentsMargins(0, 0, 0, 0)
        

class SingsSearchResultFrameBase(QFrame):

    def __init__(self, parent):
        super(SingsSearchResultFrameBase, self).__init__()
        self.parent = parent

        self.singsFrameLayout = VBoxLayout(self)

        self.noSingsContentsLabel = QLabel(self)
        self.noSingsContentsLabel.setMaximumHeight(60)

        self.noSingsContentsLabel.setObjectName("noSingsLable")
        self.noSingsContentsLabel.hide()

        self.singsResultTable = TableWidget(3, ['音乐标题', '歌手', '时长'])
        self.singsResultTable.setObjectName('singsTable')
        self.singsResultTable.setMinimumWidth(self.parent.width())
        self.singsResultTable.setColumnWidths({i:j for i,j in zip(range(3), 
            [self.parent.width()/3*1.25,self.parent.width()/3*1.25,self.parent.width()/3*0.5])})

        self.singsFrameLayout.addWidget(self.singsResultTable, Qt.AlignTop|Qt.AlignCenter)

        self.centerLabelLayout = HBoxLayout()
        self.centerLabelLayout.addStretch(1)
        self.centerLabelLayout.addWidget(self.noSingsContentsLabel)
        self.centerLabelLayout.addStretch(1)

        self.singsFrameLayout.addLayout(self.centerLabelLayout)


# 歌单详情页。
class DetailSings(ScrollArea):

    def __init__(self, parent=None):
        super(DetailSings, self).__init__(self)

        # self.hide()
        self.parent = parent
        self.setObjectName('detailSings')
        with open('QSS/detailSings.qss', 'r', encoding='utf-8') as f:
            self.setStyleSheet(f.read())

        self.setLabels()
        self.setButtons()
        self.setTabs()
        self.setLayouts()

    # 布局。
    def setLabels(self):
        self.picLabel = PicLabel(width=200, height=200)
        self.picLabel.setObjectName('picLabel')

        self.titleLabel = QLabel(self.frame)
        self.titleLabel.setObjectName('titleLabel')
        self.titleLabel.setWordWrap(True)
        self.titleLabel.setMaximumHeight(40)

        self.authorPic = QLabel(self.frame)
        self.authorName = QLabel(self.frame)
        self.authorName.setObjectName('authorName')
        self.authorName.setMaximumHeight(28)

        self.descriptionText = QTextEdit(self.frame)
        self.descriptionText.setReadOnly(True)
        self.descriptionText.setObjectName('descriptionText')
        self.descriptionText.setMaximumWidth(450)
        self.descriptionText.setMaximumHeight(100)
        self.descriptionText.setMinimumHeight(100)

    def setButtons(self):
        self.showButton = QPushButton("歌单")
        self.showButton.setObjectName('showButton')
        self.showButton.setMaximumSize(36, 20)

        self.descriptionButton = QPushButton(" 简介 ：")
        self.descriptionButton.setObjectName('descriptionButton')
        self.descriptionButton.setMaximumSize(36, 36)

        self.playAllButton = QPushButton("全部播放")
        self.playAllButton.setIcon(QIcon('resource/playAll.png'))
        self.playAllButton.setObjectName('playAllButton')
        self.playAllButton.setMaximumSize(90, 24)

    def setTabs(self):
        self.contentsTab = QTabWidget(self.frame)

        self.singsTable = TableWidget(3, ['音乐标题', '歌手', '时长'])
        self.singsTable.setObjectName('singsTable')
        self.singsTable.setMinimumWidth(self.width())
        self.singsTable.setColumnWidths({i:j for i,j in zip(range(3), 
            [self.width()/3*1.25,self.width()/3*1.25,self.width()/3*0.5])})

        self.contentsTab.addTab(self.singsTable, "歌曲列表")

    def setLayouts(self):
        self.mainLayout = VBoxLayout()

        self.topLayout = HBoxLayout()

        self.descriptionLayout = VBoxLayout()
        self.titleLayout = HBoxLayout()
        self.titleLayout.addWidget(self.showButton)
        self.titleLayout.addSpacing(5)
        self.titleLayout.addWidget(self.titleLabel)

        self.authorLayout = HBoxLayout()
        self.authorLayout.addWidget(self.authorPic)
        self.authorLayout.addWidget(self.authorName)
        self.authorLayout.addStretch(1)

        self.descriptLayout = HBoxLayout()
        self.descriptLayout.addWidget(self.descriptionButton)
        self.descriptLayout.addWidget(self.descriptionText)
        
        self.descriptionLayout.addSpacing(5)
        self.descriptionLayout.addLayout(self.titleLayout)
        self.descriptionLayout.addLayout(self.authorLayout)
        self.descriptionLayout.addSpacing(5)
        self.descriptionLayout.addWidget(self.playAllButton)
        self.descriptionLayout.addSpacing(10)
        self.descriptionLayout.addLayout(self.descriptLayout)

        self.topLayout.addWidget(self.picLabel)
        self.topLayout.addSpacing(18)
        self.topLayout.addLayout(self.descriptionLayout)

        self.mainLayout.addLayout(self.topLayout)
        self.mainLayout.addWidget(self.contentsTab)
        
        self.frame.setLayout(self.mainLayout)
        


class OneSing(QFrame):
    # 大量创建，这样可以省内存。
    __solts__ = ('parent', 'ggparent', 'detailFrame', 'row', 'column', 'ids',
     'picName', 'picLabel', 'nameLabel',
     'mainLayout',
     'mousePos',
     'result','catch',
     'singsIds', 'singsUrls')

    clicked = pyqtSignal(str, str)

    def __init__(self, row, column, ids=None, parent=None, picName=None):
        super(OneSing, self).__init__()

        self.setObjectName('oneSing')
        # 自己的位置信息。
        self.row = row
        self.column = column
        # 歌单号。
        self.ids = str(ids)
        # 大图的缓存名。
        self.picName = picName

        self.setMinimumSize(180, 235)

        self.picLabel = QLabel()
        self.picLabel.setObjectName('picLabel')
        self.picLabel.setMinimumSize(180, 180)
        self.picLabel.setMaximumSize(180, 180)

        self.nameLabel = QLabel()
        self.nameLabel.setMaximumWidth(180)
        self.nameLabel.setWordWrap(True)

        self.mainLayout = QVBoxLayout(self)

        self.mainLayout.addWidget(self.picLabel)
        self.mainLayout.addWidget(self.nameLabel)

    # 功能。
    def setStyleSheets(self, styleSheet=None):
        if styleSheet:
            self.setStyleSheet(styleSheet)

    # 事件。
    def mousePressEvent(self, event):
        # 记录下当前鼠标的位置。
        self.mousePos = QCursor.pos()

    def mouseReleaseEvent(self, event):
        # 先进行判断，防止误点将鼠标移开后还是会判断为已经点击的尴尬。
        if QCursor.pos() != self.mousePos:
            return
        else:
            self.clicked.emit(self.ids, self.picName)


class PlaylistButton(QPushButton):
    """
        提供一个简单点击的歌单自动切换点击状态，点击时会发出hasClicked信号。
        Args:
        parent 父类。
        ids 歌单的ids。
        coverImgUrl 这个歌单应该包含的图片地址 -> None, url.
        *args其他作用于原生Button的参数。
        singsIds 与 singsUrls暂时无用，不提供接受接口。

    """
    __solts__ = ('parent', 'grandparent', 'ids', 'coverImgUrl', 
        'catch', 'detailFrame', 'result', 'singsIds', 'singsUrls'
        )

    hasClicked = pyqtSignal(int, str)

    def __init__(self, parent, ids, coverImgUrl, *args):
        super(PlaylistButton, self).__init__(*args)
        self.parent = parent
        self.grandparent = self.parent.parent
        
        self.setCheckable(True)
        self.setAutoExclusive(True)

        self.ids = ids
        self.coverImgUrl = coverImgUrl

        self.catch = None
        self.result = None
        self.singsIds = None
        self.singsUrls = None

        self.clicked.connect(self.clickedEvent)

    def clickedEvent(self):
        self.hasClicked.emit(self.ids, self.coverImgUrl)



from PyQt5.QtWidgets import QLineEdit, QHBoxLayout, QPushButton, QSpacerItem, QSizePolicy, QApplication
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QCursor

"""处理时间，处理成00:00:00的格式。"""
def deal_time(x):
    x = str(x)
    if len(x) == 1:
        x = '0' + x

    return x

def itv2time(iItv):
    iItv = int(iItv)

    # 地板除求小时整数。
    h = iItv//3600
    # 求余数。
    h_remainder = iItv % 3600

    # 地板除求分钟整数。
    m = h_remainder // 60
    # 求余数 为秒。
    s = h_remainder % 60

    return ":".join(map(deal_time,(m,s)))


class SearchLineEdit(QLineEdit):
    """创建一个可自定义图片的输入框。"""
    def __init__(self, parent=None):
        super(SearchLineEdit, self).__init__()
        self.setObjectName("SearchLine")
        self.parent = parent
        self.setMinimumSize(218, 20)
        with open('QSS/searchLine.qss', 'r') as f:
            self.setStyleSheet(f.read())

        self.button = QPushButton(self)
        self.button.setMaximumSize(13, 13)
        self.button.setCursor(QCursor(Qt.PointingHandCursor))

        self.setTextMargins(3, 0, 19, 0)

        self.spaceItem = QSpacerItem(150, 10, QSizePolicy.Expanding)

        self.mainLayout = QHBoxLayout()
        self.mainLayout.addSpacerItem(self.spaceItem)
        # self.mainLayout.addStretch(1)
        self.mainLayout.addWidget(self.button)
        self.mainLayout.addSpacing(10)
        self.mainLayout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.mainLayout)
    
    def setButtonSlot(self, funcName):
        self.button.clicked.connect(funcName)



if __name__ == '__main__':
    # import sys

    # app = QApplication(sys.argv)

    # main = SearchLineEdit()

    # main.show()

    # sys.exit(app.exec_())
    print(itv2time(12.34))


import os
import os.path

from dbManager import DbManager

home = os.path.expanduser('~')

if '.musicplayer' not in os.listdir(home):
    os.mkdir(os.path.join(home, '.musicplayer'))

dbPath = os.path.join(home, '.musicplayer', 'userData.db')

createTableString = """
CREATE TABLE IF NOT EXISTS playhistory(
    id INTEGER PRIMARY KEY autoincrement,
    name TEXT,
    author TEXT,
    playTimes INTEGER default 1    
)"""


# 粗略设计
# 所有平台的id号都归为一类，
# 但推荐时使用的网易云系统。
class UserDbManager(DbManager):

    def __init__(self, database=dbPath, *args):
        super().__init__(database, *args)
        self.initDb()

    def initDb(self):
        self.createTable(createTableString)

    def getSongByPlayTimes(self, maxes=5) -> list:
        """
            返回前5个播放次数最多的歌曲信息。
        """
        fetchedData = self.cursor.execute("""SELECT id FROM playhistory 
            ORDER BY playTimes DESC 
            LIMIT {}""".format(maxes))

        # TODO
        # 错误过滤
        return [i[0] for i in fetchedData]

    def addPlayTimesById(self, songId, name='Test', author='Test'):
        """
            给songId所标示的数据playTimes + 1
        """
        try:
            insertData = self.cursor.execute("""INSERT INTO playhistory 
                (id, name, author) VALUES 
                ({0}, '{1}', '{2}')
                """.format(songId, name, author))
        except:
            # 暂时不知道sqlite中如何进行
            # 无此数据时创建，有此数据时将某个值在其基础上增加或减少。
            # 目前的思路是在初始化时进行一次查询，然后将数据次数都保存在内存中进行增减，
            # 这样可以减少数据库操作。
            # 但数据量较小，不是主要考虑目标。

            updateData = self.cursor.execute("""UPDATE playhistory 
                SET playTimes = playTimes + 1
                WHERE id = {}
                """.format(songId))

        self.db.commit()

if __name__ == '__main__':
    
    userDb = UserDbManager()

    # userDb.addPlayTimesById(7, '5', '6')
    userDb.cursor.execute("""SELECT * from playhistory""")
    print(userDb.cursor.fetchall())
    for i in userDb.cursor.fetchall():
        print(i)

    # print(userDb.getSongByPlayTimes())


"""用于定义几个需要多次调用的基础类。"""
__author__ = 'cyrbuzz'

from queue import Queue

# 这是一个次级目录。
import os
import sys
# sys.path.append('..')
# sys.path.append('../networks')
import pickle
import hashlib
import os.path
import logging

# PEP8不允许使用通配符的原因是会混淆命名空间。
# PyQt5的所有命名都是QXXX, 这边暂时不改了。
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

from network import Requests


logger = logging.getLogger(__name__)


# 保存cookies的基础装饰器用于检测是否存在这个文件。
# 保存为pickle.
def checkFolder(filenames:iter):
    # /test/test/file.suffix
    for filename in filenames:
        splits = filename.split('/')
        # 检查目录是否存在.
        for i in range(len(splits[:-1])):
            dirs = '/'.join(splits[:i+1])
            if not os.path.isdir(dirs):
                os.mkdir(dirs)

        if not os.path.isfile(filename):
            with open(filename, 'wb') as f:
                pass

    def _check(func):
        def _exec(*args):
            try:
                func(*args)
            except:
                logger.warning('读取或保存cookies出错 文件名: {0}'.format(filenames), exc_info=True)
                print('读取或保存cookies出错', filenames)

        return _exec
    return _check


def checkOneFolder(folderName:str):
    if not os.path.isdir(folderName):
        os.mkdir(folderName)

    def _check(func):
        def _exec(*args):
            try:
                func(*args)
            except:
                logger.warning('读取或保存cookies出错 文件夹名: {0}'.format(folderName), exc_info=True)
                print('读取或保存cookies出错', folderName)

        return _exec
    return _check


def makeMd5(raw):

     m = hashlib.md5()  
     m.update(raw.encode()) 
     return  m.hexdigest()  


# Qt的一些部件是直接支持HTML的，
# 做个用于方便支持居中的装饰器。
def centerHTML(func):
    def wrap(*args):

        centerHTML = '<p align="center" style="line-height: 175%">{0}</p>'.format(func(*args).replace('\n', '<br>'))

        return centerHTML

    return wrap


# 一个有信号槽机制的安全线程队列。
class QueueObject(QObject):
    add = pyqtSignal()

    def __init__(self):
        super(QueueObject, self).__init__()
        self.queue = Queue()

    def put(self, data):
        self.queue.put(data)
        self.add.emit()

    def get(self):
        if self.queue.empty():
            return 0

        return self.queue.get()


# 一个用于继承的类，方便多次调用。
class ScrollArea(QScrollArea):
    """包括一个ScrollArea做主体承载一个QFrame的基础类。"""
    scrollDown = pyqtSignal()

    def __init__(self, parent=None):
        super(ScrollArea, self).__init__()
        self.parent = parent
        self.frame = QFrame()
        self.frame.setObjectName('frame')
        # 用于发出scroll滑到最底部的信号。
        self.verticalScrollBar().valueChanged.connect(self.sliderPostionEvent)

        self.setWidgetResizable(True)

        self.setWidget(self.frame)

    def noInternet(self):
        # 设置没有网络的提示。
        self.noInternetLayout = QGridLayout()
        self.setLayout(self.mainLayout)

        self.Tip = QLabel("您已进入没有网络的异次元，打破次元壁 →", self)
        self.TipButton = QPushButton("打破次元壁", self)
        self.TipButton.setObjectName("TipButton")

        self.TipLayout = QHBoxLayout()
        self.TipLayout.addWidget(self.Tip)
        self.TipLayout.addWidget(self.TipButton)

        # self.indexAllSings.setLayout(self.TipLayout)

        self.noInternetLayout.addLayout(self.TipLayout, 0, 0, Qt.AlignCenter|Qt.AlignTop)

        self.frame.setLayout(self.noInternetLayout)

    def sliderPostionEvent(self):
        if self.verticalScrollBar().value() == self.verticalScrollBar().maximum():
            self.scrollDown.emit()

    def maximumValue(self):
        return self.verticalScrollBar().maximum()


class TableWidget(QTableWidget):

    def __init__(self, count, headerLables):
        super(TableWidget, self).__init__()
        self.setColumnCount(count)
        self.setHorizontalHeaderLabels(headerLables)

        self.horizontalHeader().setStretchLastSection(True)
        self.verticalHeader().setVisible(False)
        self.setShowGrid(False)
        self.setAlternatingRowColors(True)
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)

    def setColumnWidths(self, widths:dict):
        for key in widths:
            self.setColumnWidth(key, widths[key])


# 去除了margin和spacing的布局框。
class VBoxLayout(QVBoxLayout):

    def __init__(self, *args):
        super(VBoxLayout, self).__init__(*args)

        self.setContentsMargins(0, 0, 0, 0)
        self.setSpacing(0)


class HBoxLayout(QHBoxLayout):

    def __init__(self, *args):
        super(HBoxLayout, self).__init__(*args)

        self.setContentsMargins(0, 0, 0, 0)
        self.setSpacing(0)


# 默认情况下。
# ----!!!----
# 一个水平居中的布局。
class HStretchBox(HBoxLayout):

    def __init__(self, parentLayout, *widgets, frontStretch=1, behindStretch=1):
        super(HStretchBox, self).__init__()
        self.addStretch(frontStretch)
        for i in widgets:
            self.addWidget(i)

        self.addStretch(behindStretch)

        parentLayout.addLayout(self)


# 默认情况下。
#  |
#  !
#  |
# 一个垂直居中的布局。
class VStretchBox(VBoxLayout):

    def __init__(self, parentLayout, *widgets, frontStretch=1, behindStretch=1):
        super(VStretchBox, self).__init__()
        self.addStretch(frontStretch)
        for i in widgets:
            self.addWidget(i)
        self.addStretch(behindStretch)

        self.parentLayout.addLayout(self)


# ---下面是线程，包括网络线程和时钟线程。
class RequestThread(QThread):
    # 该信号用于在线程执行时发出，通知主线程做些界面更新。
    # 因为在其他线程修改主界面是非常危险的事情。
    breakSignal = pyqtSignal(str)

    """异步请求，类似Pyhton封装的Thread形式，用QThread在简单封装一下。"""
    def __init__(self, parent=None, target=None, finishedConnection=None, *args, **kwargs):
        super(RequestThread, self).__init__()

        self.parent = parent
        self.args = args
        self.kwargs = kwargs
        self.target = target
        self.flag = False
        self.result = None
        if finishedConnection:
            self.finished.connect(finishedConnection)
            
    def run(self):
        self.result = self.target(*self.args, **self.kwargs)

    def setTarget(self, target=None):
        """方便多次调用。"""
        self.target = target

    def setArgs(self, *args, **kwargs):
        """方便多次调用。"""
        self.args = args
        self.kwargs = kwargs

    def setFlag(self, flag):
        self.flag = flag


class Timer(QThread):
    # 一个无限循环的时钟，只有特定的条件才会解锁。
    def __init__(self, var=None, parent=None):
        super(Timer, self).__init__()
        self.parent = parent
        self.var = var
        self.times = 0

    def run(self):
        while 1:
            if self.var:
                self.var = False
                break
            self.sleep(0.1)

    def setVar(self, value):
        """设置变量。"""
        self.var = value


# <img src=1.jpg>相关变量。

picsThreadPool = QThreadPool()
picsThreadPool.setMaxThreadCount(10)

picsQueue = QueueObject()

# 缓存目录。
cacheFolder = 'cache'


## 对<img src=1.jpg>的初步探索。
# 暂只接受http(s)和本地目录。
class PicLabel(QLabel):

    def __init__(self, src=None, width=200, height=200, pixMask=None):
        super(PicLabel, self).__init__()
        global picsThreadPool

        self.src = None

        self.width = width
        self.height = height

        self.pixMask = None
        if pixMask:
            self.pixMask = pixMask
        if src:
            self.setSrc(src)

        if self.width:
            self.setMaximumSize(self.width, self.height)
            self.setMinimumSize(self.width, self.height)

    @checkOneFolder(cacheFolder)
    def setSrc(self, src):
        src = str(src)
        if 'http' in src or 'https' in src:
            cacheList = os.listdir(cacheFolder)

            # names = str(src[src.rfind('/')+1:])
            names = makeMd5(src)
            localSrc = cacheFolder+'/'+names
            if names in cacheList:
                self.setSrc(localSrc)
                self.src = localSrc
                return

            task = GetPicture(self, src)
            picsThreadPool.start(task)
        else:
            self.src = src
            pix = QPixmap(src)
            pix.load(src)
            pix = pix.scaled(self.width, self.height)
            # mask需要与pix是相同大小。
            if self.pixMask:
                mask = QPixmap(self.pixMask)
                mask = mask.scaled(self.width, self.height)
                pix.setMask(mask.createHeuristicMask())

            self.setPixmap(pix)

    def getSrc(self):
        """返回该图片的地址。"""
        return self.src


class GetPicture(QRunnable):

    def __init__(self, widget, src):
        super(GetPicture, self).__init__()
        global picsQueue
        self.widget = widget
        self.src = src

    def run(self):
        # names = str(self.src[self.src.rfind('/')+1:])
        names = makeMd5(self.src)
        content = Requests.get(self.src).content
        picsQueue.put([self.widget, content, names])


# just for picLabel.
def __addPic():
    data = picsQueue.get()
    # widget, content, names.
    if not data:
        return

    widget = data[0]
    width = widget.width
    height = widget.height

    pic = QPixmap()
    pic.loadFromData(data[1])
    localSrc = cacheFolder+'/'+data[2]
    pic.save(localSrc, 'jpg')
    pic = pic.scaled(width, height)

    widget.src = localSrc

    # 上遮罩。
    if widget.pixMask:
        mask = QPixmap()
        mask.load(widget.pixMask)
        mask = mask.scaled(width, height)

        pic.setMask(mask.createHeuristicMask())
    
    widget.setPixmap(pic)


picsQueue.add.connect(__addPic)


# 图片模糊化处理函数。
try:
    # from PIL import Image, ImageFilter
    noPIL = False
except ImportError:
    noPIL = True


# 本来想做图片虚化然后显示的效果，
# 发现显示出来比较丑。
def blur(source):
    return False

# def blur(source:str):
#     """接受图片路径，返回处理后的图片路径。"""
#     if noPIL:
#         return False

#     image = Image.open(source)
#     image = image.filter(ImageFilter.GaussianBlur(radius=30))
#     saveName = source + '_blur.jpg'
#     image.save(saveName)

#     return saveName


if __name__ == '__main__':
    import os
    os.chdir('..')

    app = QApplication([])

    # a = QFrame()
    a = PicLabel('F:/pics/Crow.jpg', width=64, height=64)
    # a.setObjectName('sss')
    # a.setStyleSheet('QLabel#sss {border-image: url(F:/pics/Crow.jpg);}')
    # a.resize(500, 600)
    b = PicLabel('resource/expand.png', 64, 64)
    b.setStyleSheet('QLabel {background-color: rgba(0, 0, 0,50%);}')
    c = VBoxLayout(a)
    c.addWidget(b)
    a.show()

    exit(app.exec_())


__author__ = 'cyrbuzz'

"""
提供用于将函数包装成异步的基本功能。
"""
import asyncio


def aAsync(func, *args, **kwargs):
    """
    future 用于将任意函数包装成一个awaitable对象。
    例：
    future = aAsync(requests.get, 'http://www.xxx.com', headers=headers)
    data = yield from future
    """
    
    # run_in_evecutor不支持**kwargs.
    # args是个元组，kwargs是个字典。
    # 再用到时 run_in_evecutor(None, makeUp, args, kwargs)
    # 注意这边不要带*，带*为解包，不要解包。
    def makeUp(args, kwargs):
        return func(*args, **kwargs)

    eventLoop = asyncio.get_event_loop()
    future = eventLoop.run_in_executor(None, makeUp, args, kwargs)

    return future


def toTask(func):
    """
        一个将普通函数包装成异步函数的装饰器。
        例：
        @toTask
        def test(x):

            future = aAsync(requests.get, 'http://www.xxx.com', headers=headers)
            print(x)
            data = yield from future

            print(data)

        在运行test时(test())就会变成一个异步函数，
        里面I/O部分就会由asyncio提供的事件循环处理。
        >>> for i in range(5):
                test(x)
        ```
        0
        1
        2
        3
        4
        <Response [200]>
        <Response [200]>
        <Response [200]>
        <Response [200]>
        <Response [200]>
        ```
    """
    def makeUp(*args, **kwargs):
        eventLoop = asyncio.get_event_loop()
        future = eventLoop.create_task(func(*args, **kwargs))

        return future

    return makeUp


def toTaskWCb(func):
    """
    一个将普通函数包装成异步函数并添加回调的装饰器。
    """
    def makeUp(callback):
        def makeUps(*args, **kwargs):
            eventLoop = asyncio.get_event_loop()
            future = eventLoop.create_task(func(*args, **kwargs))
            
            future.add_done_callback(callback)

            return future
        
        return makeUps

    return makeUp


if __name__ == '__main__':
    help(aAsync)
    print('\n')
    help(toTask)


"""
界面与功能分离的初步尝试。
"""
__author__ = 'cyrbuzz'
import os
import re

import network
import addition

from base import (QAction, QCursor, QFrame, QLabel, QObject, QPixmap, QRunnable, RequestThread, 
                                     QMenu, QTableWidgetItem, QThreadPool, QueueObject, makeMd5, pyqtSignal)
from netEaseApi import netease
from singsFrameBase import OneSing, PlaylistButton

# ../features
from asyncBase import aAsync, toTask

# ../apis
from netEaseApi import netease
from apiRequestsBase import HttpRequest

myRequests = HttpRequest()


transTime = addition.itv2time


class ConfigNetEase(QObject):

    def __init__(self, parent=None):
        super(ConfigNetEase, self).__init__()
        self.netEase = parent
        self.netEaseParent = self.netEase.parent
        # window - > detailSings.
        self.detailFrame = self.netEaseParent.parent.detailSings
        self.mainContents = self.netEaseParent.parent
        # ThreadPool方式。
        # 线程池方法说明:
        # PyQt的线程池由此创建，最好指定下最大连接数。
        # QThreadPool需要一个QRunnable对象作为目标。
        # 如下所创建的_PicThreadTask，需要重写run函数。
        # 然后在需要使用时创建:
        # task = _PicThreadTask
        # 并交由QThreadPool开始，self.picThreadPool(task)
        # 循环就可以。
        # 由于QRunnable不是由QObject继承来的，所以无法享受到信号槽机制。
        # 而PyQt中跨线程最好是不要进行界面操作。否则会有非常多意想不到的后果。
        self.picThreadPool = QThreadPool()
        self.picThreadPool.setMaxThreadCount(5)

        # QueueObject定义在base中，是一个由Queue与QObject组成的对象。
        # 一方面Queue线程安全，另一方面QObject带有信号槽机制。
        # 那只要QRunnable线程中请求完了内容，将内容添加到QueueObject中，
        # 由QueueObject发出信号通知主线程进行界面操作就可以安全的完成。
        # 这里是图片的操作。
        self.queue = QueueObject()
        self.queue.add.connect(self._setStyleCodesByThreadPool)

        # 连接滑轮到底的信号槽。
        # 同时连接图片下载的线程全部完成的信号槽。
        # 若一轮图片下载完成并且滑到底部则进行下一次线程，否则将不会。
        self.netEase.scrollDown.connect(self.sliderDownEvent)
        # self.picManager.allFinished.connect(self.picManagerFinishedEvent)
        
        # 用于存储结果。
        self.result = []

        # 歌单请求后的内容和缓存。
        self.reqResult = None
        self.cache = None
        self.singsUrls = None
        self.picName = None

        # 歌单的索引。
        self.singsFrames = []
        
        # 歌单显示名的url。
        self.singPicUrls = []
        
        # 歌单名称。
        self.singNames = []
        
        # 歌单id。
        self.playlistIds = []

        # 歌曲ids。
        self.singsIds = []

        # 一个是否滑到底部的flag。
        self.sliderDown = False

        # 布局用row。
        self.gridRow = 0

        # 布局用column。
        self.gridColumn = 0

        self.offset = 0
        # 用于不足时的补足。
        self.offsetComplement = 30

        self.myHeight = 0

        self.api = netease

        # self.initThread()

    def initThread(self):
        # 一个线程，初始化用于请求歌单的全部内容。
        self.netThread = RequestThread(self, self.getSings)
        self.netThread.finished.connect(self.threadSetSings)

        self.netThread.setFlag(True)
        self.netThread.start()

        self.singsThread = RequestThread(self)
        self.singsThread.setTarget(self.requestsDetail)
        self.singsThread.finished.connect(self.setRequestsDetail)

    def getSings(self):
        """请求一波歌单，一次30个。设置offset会设置请求量。"""
        result = self.api.all_playlist(offset=self.offset)
        if not result:
            return 

        for i in result:
            self.result.append(i)
            self.singNames.append(i['name'])
            self.singPicUrls.append(i['coverImgUrl'])
            self.playlistIds.append(i['id'])
  
    """测试线程池方法。"""
    def threadSetSings(self):
        if not self.result:
            return 
        length = len(self.singPicUrls)

        for i in range(30):
            i += self.offset
            # 根本原因是不足30个。
            if i >= length:
                self.offsetComplement = length % 30 
                break

            picName = makeMd5(self.singPicUrls[i])
            frame = OneSing(self.gridRow, self.gridColumn, self.playlistIds[i], self, picName)
            frame.clicked.connect(self.startRequest)
            frame.nameLabel.setText(self.singNames[i])
            
            self.netEase.mainLayout.addWidget(frame, self.gridRow, self.gridColumn)
            # 建立起索引，一是防止垃圾回收了，二是可以找到他的地址。
            self.singsFrames.append(frame)

            # 用于布局，一行4个。
            if self.gridColumn == 3:
                self.gridColumn = 0
                self.gridRow += 1
            else:
                self.gridColumn += 1

            try:
                cacheList = os.listdir('cache')
            except:
                os.mkdir('cache')
                cacheList = os.listdir('cache')
            
            url = self.singPicUrls[i]

            # names = str(url[url.rfind('/')+1:])   
            names = makeMd5(url)
            if names in cacheList:
                frame.setStyleSheets("QLabel#picLabel{border-image: url(cache/%s)}"%(names))
            else:
                task = _PicThreadTask(self.queue, frame, url)
                self.picThreadPool.start(task)
        else:
            # 如果顺利进行会将offsetComplement变成原30。
            self.offsetComplement = 30

    def _setStyleCodesByThreadPool(self):
        # data是线程池的请求完成后的对象。
        # 0下标处是widget，1是style代码。
        data = self.queue.get()
        if not data:
            return
        else:
            data[0].setStyleSheets(data[1])

    def startRequest(self, ids, picName):
        self.picName = picName
        self.singsThread.setArgs(ids)
        self.singsThread.start()

    def requestsDetail(self, ids):
        reqResult = self.api.details_playlist(ids)
        self.reqResult = reqResult

        # 网易云此处不再返回歌曲地址，由之后播放时单独获取。
        self.singsIds = [i['id'] for i in reqResult['tracks']]

        self.singsUrls = ['http{0}'.format(i) for i in enumerate(self.singsIds)]

    def setRequestsDetail(self):
        result = self.reqResult

        self.detailFrame.config.setupDetailFrames(result, self.singsUrls, self.singsIds)
        self.detailFrame.picLabel.setSrc('cache/{0}'.format(self.picName))
        self.detailFrame.picLabel.setStyleSheet('''QLabel {padding: 10px;}''')
        
        # 隐藏原来的区域，显示现在的区域。
        self.mainContents.mainContents.setCurrentIndex(1)

    # 事件。
    def sliderDownEvent(self):
        """滑轮到底的事件。"""
        if self.netEase.isHidden() == False:
        # toDo, 多个
            self.offset += self.offsetComplement
            # 判断是否在工作，免得多次start。
            if self.netThread.isRunning():
                return
            else:
                self.netThread.start()


class _PicThreadTask(QRunnable):
    # finished = pyqtSignal(QFrame, str)
    def __init__(self, queue, widget=None, url=None):
        super(_PicThreadTask, self).__init__()
        self.queue = queue
        self.widget = widget
        self.url = url

    def run(self):
        names = makeMd5(self.url)
        content = network.Requests.get(self.url).content
        pic = QPixmap()
        pic.loadFromData(content)
        # 缩小到合适的大小会让QT合理的利用内存资源。
        pic = pic.scaled(180, 180)
        a = pic.save("cache/{0}".format(names), 'jpg')
        self.queue.put([self.widget, "QLabel#picLabel{border-image: url(cache/%s)}"%(names)])




import requests


# TCP重传需要3秒。
default_timeout = 3.05

headers = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
    'Connection': 'keep-alive',
    'Pragma': 'no-cache',
    'Cache-Control': 'no-cache',
    'Accept-Encoding': 'gzip,deflate,sdch',
    'Accept-Language': 'zh-CN,zh;q=0.8',
    'Upgrade-Insecure-Requests': '1',
    'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/45.0.2454.101 Safari/537.36'
}

cookies = {}


def requestsExceptionFilter(func):

    def _filter(*args, **kwargs):
        for i in range(3):
            try:
                return func(*args, **kwargs)
            except:
                continue
        else:
            return False
    
    return _filter


class Requests(object):
    def __init__(self):
        self.headers = headers.copy()

    @requestsExceptionFilter
    def get(self, url, **kwargs):
        if not kwargs.get('headers'):
            kwargs['headers'] = self.headers
        return requests.get(url, **kwargs)
    
    @requestsExceptionFilter
    def post(self, url, **kwargs):
        if not kwargs.get('headers'):
            kwargs['headers'] = self.headers

        return requests.post(url, **kwargs)  


class Session(object):
    
    def __init__(self):
        self.headers = headers.copy()
        self.session = requests.session()

    @requestsExceptionFilter
    def get(self, url, **kwargs):
        if not kwargs.get('headers'):
            kwargs['headers'] = self.headers

        return self.session.get(url, **kwargs)

    @requestsExceptionFilter
    def post(self, url, **kwargs):
        if not kwargs.get('headers'):
            kwargs['headers'] = self.headers

        return self.session.post(url, **kwargs)  





# coding = utf-8

import json
import logging
import urllib.parse

from apiRequestsBase import HttpRequest, ignored


logger = logging.getLogger(__name__)


class QQApi(HttpRequest):

    default_timeout = 3.05

    def __init__(self):
        super(QQApi, self).__init__()

        self.headers['Host'] = 'c.y.qq.com'
        self.headers['Referer'] = 'https://y.qq.com/portal/playlist.html'

        self.playlistHeaders = self.headers.copy()
        self.playlistHeaders['Host'] = 'shc.y.qq.com'

        self.tokenHeaders = self.headers.copy()
        self.tokenHeaders['Host'] = 'base.music.qq.com'
        self.tokenHeaders.pop('Referer')

        self.token = self._get_qqtoken()
        self.key = self.token.get('key')
        self.sip = self.token.get('sip')[0]
        # 随便写一个就可以，原本是根据cookies里一个根据时间变化的参数确定的。
        self.guid = 3768717388
        if not self.sip:
            logger.info("获取QQToken失败。当前key: {0}, 当前sip: {1}".format(self.key, self.sip))
            print('QQ 播放地址获取失败，请勿播放QQ音乐。')

    def httpRequest(self, *args, **kwargs):
        html = super(QQApi, self).httpRequest(*args, **kwargs)

        logger.info("进行QQ Url请求, args: {0}, kwargs: {1}".format(args, kwargs))
        with ignored():
            return html.text

        logger.info("url: {0} 请求失败. Header: {1}".format(args[0], kwargs.get('headers')))
        return ''

    # copy from listen1.
    # https://github.com/listen1/listen1/blob/master/listen1/replay/qq.py
    def _get_qqtoken(self):
        token_url = 'http://base.music.qq.com/fcgi-bin/fcg_musicexpress.fcg?' + \
            'json=3&guid=3768717388&g_tk=938407465&loginUin=0&hostUin=0&' + \
            'format=jsonp&inCharset=GB2312&outCharset=GB2312&notice=0&' + \
            'platform=yqq&jsonpCallback=jsonCallback&needNewCode=0'
        
        data = self.httpRequest(token_url, method='GET', headers=self.tokenHeaders)
        
        with ignored():
            data = data[len("jsonCallback("):-len(");")]

            return json.loads(data)
        return {'key': '1', 'sip': ['']}

    def _getImgUrl(self, mid):
        imgUrl = 'https://y.gtimg.cn/music/photo_new/'
        return imgUrl + 'T002R300x300M000' + mid + '.jpg'

    def _getSongUrl(self, mid):
        return '{0}C400{1}.m4a?vkey={2}&guid={3}'.format(self.sip, mid, self.key, self.guid)

    def playList(self, ein=29):
        """
        ein控制返回的歌单。
        29, 59, 89....
        """
        url = 'https://c.y.qq.com/splcloud/fcgi-bin/'+\
        'fcg_get_diss_by_tag.fcg?rnd=0.5136307078685405&g_tk=5381&'+\
        'jsonpCallback=getPlaylist&loginUin=0&hostUin=0&format=jsonp&inCharset=utf8'+\
        '&outCharset=utf-8&notice=0&platform=yqq&needNewCode=0&categoryId=10000000&'+\
        'sortId=5&sin=30&ein={0}'.format(ein)

        response = self.httpRequest(url, method='GET', headers=self.headers)

        with ignored():
            data = json.loads(response[len('getPlaylist('):-len(')')])
        
            return data['data']['list']
        
        return False

    def getPlaylist(self, ids):
        url = 'https://shc.y.qq.com/qzone/fcg-bin/fcg_ucc_getcdinfo_byids_cp.fcg?type=1&json=1&utf8=1&onlysong=0'+\
        '&disstid={0}&format=jsonp&g_tk=5381&jsonpCallback=playlistinfoCallback&loginUin=0&hostUin=0&'.format(ids)+\
        'format=jsonp&inCharset=utf8&outCharset=utf-8&notice=0&platform=yqq&needNewCode=0'
        
        response = self.httpRequest(url, method='GET', headers=self.playlistHeaders)
        with ignored():
            data = json.loads(response[len('playlistinfoCallback('):-len(')')])
            data = data['cdlist'][0]

            newDatas = {}
            newDatas['trackCount'] = data['total_song_num']
            newDatas['name'] = data['dissname']
            newDatas['creator'] = {'nickname': data['nick']}
            newDatas['description'] = data['desc']
            songs = data['songlist']

            # imgUrl = 'https://y.gtimg.cn/music/photo_new/'

            for i in songs:
                i['name'] = i['songname']
                i['artists'] = [{'name': ';'.join([x['name'] for x in i['singer']])}]
                i['duration'] = int(i['interval']) * 1000
                # i['album'] = {'blurPicUrl': imgUrl + 'T002R300x300M000' + i['albummid'] + '.jpg'}
                i['album'] = {'blurPicUrl': self._getImgUrl(i['albummid'])}
                # i['mp3Url'] = '{0}C400{1}.m4a?vkey={2}&guid={3}'.format(self.sip, i['songmid'], self.key, self.guid)
                i['mp3Url'] = self._getSongUrl(i['songmid'])
                i['lyric'] = 'qq'

            newDatas['tracks'] = songs

            return newDatas

        return False

    def search(self, key):
        url = 'https://c.y.qq.com/soso/fcgi-bin/client_search_cp?ct=24&qqmusic_ver=1298&'+\
                   'new_json=1&remoteplace=txt.yqq.center&searchid=43541888870417375&t=0&aggr=1'+\
                   '&cr=1&catZhida=1&lossless=0&flag_qc=0&p=1&n=50&'+\
                   'w={0}'.format(urllib.parse.quote(key))+\
                   '&g_tk=5381&jsonpCallback=searchCallbacksong6064&loginUin=0&hostUin=0&'+\
                   'format=jsonp&inCharset=utf8&outCharset=utf-8&notice=0&platform=yqq&needNewCode=0'

        response = self.httpRequest(url, method='GET')

        with ignored():
            data = json.loads(response[len('searchCallbacksong6064('):-len(')')])
            
            data = data['data']['song']

            newDatas = {}
            newDatas['songCount'] = data['curnum'] - 1
            songs = []
            for i in data['list']:
                songs.append({ 'name': i['name'],
                                                  'ar': [{'name': ';'.join([x['name'] for x in i['singer']])}],
                                                  'al': {'picUrl': self._getImgUrl(i['album']['mid'])},
                                                  'dt': i['interval'] * 1000,
                                                  'id': i['id'],
                                                  # 当然这里不是mp3，为了统一接口这样写。
                                                  'mp3Url': self._getSongUrl(i['mid']),
                                                  'lyric': 'qq'
                    })

            newDatas['songs'] = songs
            
            return newDatas 

        return False

qqApi = QQApi()


if __name__ == '__main__':
    pass
    # a = qqApi.playList(29)
    # http://dl.stream.qqmusic.qq.com/
    # C400 0000HZ4N01bJ8a.m4a
    #           0000HZ4N01bJ8a
    # ?vkey=42BBA28D626228E8C360FEFE03817E3A15885426F969484963ABFBC4D197A5CEF4E3ED7911F3236150212564FA9F2F1B0AD02AF37FF181A2&guid=3768717388&uin=0&fromtag=66
    # ?vkey=1B6C73D1685155EBB75D36995A81FD2DEFEA2BA30389B99675EDDB367BF4FFCFC075B63FBFFA9F4642A411C02A248FEF24E91379DBDEBA7D&guid=780782017&uin=0&fromtag=66
    # a = qqApi._get_qqtoken()
    # print(qqApi.key, qqApi.sip)    
    a = qqApi.search("故梦")
    print(a)
    for i in a:
        print(i)
    #     print(a[i])
    # print(len(a['list']))
        # print(i['d
        # issname'])
        # print(i['dissid'])

    # for i in a:
        # print(i)
    #     print(a[i])


# 加密参数相关。
# 具体http://s3.music.126.net/sep/s/2/core.js?5d6f8e4d01b4103ec9f246a2ef70e6d1在这个js中可以查看。
# 好吧，其实不用分析这个js，在这个git中可以找到https://github.com/xiyouMc/ncmbot，不过他是for python2的。
# 针对pyton3做了修改。

import os
import json
import base64
import random
import hashlib
import binascii

try:
    from Cryptodome.Cipher import AES
except:
    from Crypto.Cipher import AES
    
modulus = '00e0b509f6259df8642dbc35662901477df22677ec152b5ff68ace615bb7b725152b3ab17a876aea8a5aa76d2e417629ec4ee341f56135fccf695280104e0312ecbda92557c93870114af6c9d05c4f7f0c3685b7a46bee255932575cce10b424d813cfe4875d3e82047b97ddef52741d546b8e289dc6935b3ece0462db0a22b8e7'
nonce = b'0CoJUm6Qyw8W8jud'
pubKey = '010001'


def aesEncrypt(text, secKey):

    pad = 16 - len(text) % 16

    # aes加密需要byte类型。
    # 因为调用两次，下面还要进行补充位数。
    # 直接用try与if差不多。

    try:
        text = text.decode()
    except:
        pass

    text = text + pad * chr(pad)
    try:
        text = text.encode()
    except:
        pass

    encryptor = AES.new(secKey, 2, bytes('0102030405060708', 'utf-8'))
    ciphertext = encryptor.encrypt(text)
    ciphertext = base64.b64encode(ciphertext)
    return ciphertext


def createSecretKey(size):
    # 2中 os.urandom返回是个字符串。3中变成bytes。
    # 不过加密的目的是需要一个字符串。
    # 因为密钥之后会被加密到rsa中一起发送出去。
    # 所以即使是个固定的密钥也是可以的。

    # return (''.join(map(lambda xx: (hex(ord(xx))[2:]), os.urandom(size))))[0:16]
    return bytes(''.join(random.sample('1234567890qwertyuipasdfghjklzxcvbnm', 16)), 'utf-8')


def rsaEncrypt(text, pubKey, modulus):
    text = text[::-1]
    # 3中将字符串转成hex的函数变成了binascii.hexlify, 2中可以直接 str.encode('hex')
    rs = int(binascii.hexlify(text), 16) ** int(pubKey, 16) % int(modulus, 16)

    return format(rs, 'x').zfill(256)


def encrypted_request(text):

    # 这边是加密过程。
    text = json.dumps(text)
    secKey = createSecretKey(16)
    encText = aesEncrypt(aesEncrypt(text, nonce), secKey)
    encSecKey = rsaEncrypt(secKey, pubKey, modulus)
    # 在那个js中也可以找到。

    # params加密后是个byte，解下码。
    data = {
        'params': encText.decode(),
        'encSecKey': encSecKey
    }
    return data




"""
    提供封装好的持久连接的Http请求类。
    默认返回(cookies, jsonData),
    is_json参数为False时返回(cookies, text),
    不提供返回二进制。
"""

__author__ = 'cyrbuzz'

import logging
import requests

from contextlib import contextmanager


logger = logging.getLogger(__name__)


# 不应该在这定义。
# 暂且放在这。
@contextmanager
def ignored(*exception):
    """
    使用上下文管理的方式忽略错误。
    with ignored(OSError):
        print(1)
        raise(OSError)

    print(2)

    """
    if exception:
        try:
            yield
        except exception:
            logger.error("error has ignored.", exc_info=True)
    else:
        try:
            yield 
        except:
            logger.error("error has ignored.", exc_info=True)
            


def requestsExceptionFilter(func):
    """
    若某一函数出错(一般是网络请求), 会再次进行2次重新请求，否则会传回False
    @requestsExceptionFilter
    def test():
        requests.get('http://www.thereAreNothing.com')
    
    test()
    ---
    False
    """
    def _filter(*args, **kwargs):
        for i in range(3):
            try:
                return func(*args, **kwargs)
            except:
                logger.error("retry function {0} args {1}, kwargs {2} times:{3}".format(func, args, kwargs, i))
                continue
        else:
            logger.error("function {0} is wrong. args {1}, kwargs {2}".format(func, args, kwargs))
            return False
    
    return _filter


class HttpRequest(object):
    # 使用keep-alive，
    # keep-alive保持持久连接，没有必要开启很多个TCP链接，浪费资源。
    # 使用会话(session)来保持持久连接。
    # sessions = requests.session()
    # cookies也可以方便管理。
    
    # TCP重传需要3秒。
    default_timeout = 3.05
    
    headers = {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        'Connection': 'keep-alive',
        'Pragma': 'no-cache',
        'Cache-Control': 'no-cache',
        'Accept-Encoding': 'gzip,deflate,sdch',
        'Accept-Language': 'zh-CN,zh;q=0.8',
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/45.0.2454.101 Safari/537.36'
    }

    cookies = {}

    def __init__(self):
        self.sessions = requests.session()
        self.headers = self.headers.copy()

    @requestsExceptionFilter
    def httpRequest(self, action, method="GET", add=None, data=None, headers=None, cookies='',\
                    timeout=default_timeout, urlencode='utf-8', is_json=True):
        """
            默认以get方式请求，
            GET方式附加内容用add参数，POST方式提交内容用data参数。
            编码用urlencode参数，默认utf-8。
            默认cookies为空。
        """
        if not headers:
            headers = self.headers

        if method.upper() == 'GET':
            if add:
                html = self.sessions.get(action, params=add, headers=headers, cookies=cookies, timeout=timeout)
            else:
                html = self.sessions.get(action, headers=headers, cookies=cookies, timeout=timeout)
            html.encoding = urlencode

        elif method.upper() == 'POST':
            if data:
                html = self.sessions.post(action, data=data, headers=headers, cookies=cookies, timeout=timeout)
            else:
                html = self.sessions.post(action, headers=headers, cookies=cookies, timeout=timeout)
            html.encoding = urlencode

        return html

    def __del__(self):
        # 关闭请求。
        with ignored():
            self.sessions.close()


if __name__ == '__main__':
    # a = requests.session()
    help(ignored)
    print('\n')
    help(requestsExceptionFilter)
    print('\n')
    help(HttpRequest)


__author__ = 'cyrbuzz'

from base import QAction, QIcon, QMenu, QSystemTrayIcon


class SystemTray(QSystemTrayIcon):
    
    def __init__(self, iconPath, parent=None):
        super(SystemTray, self).__init__(QIcon(iconPath))
        self.parent = parent

        self.menu = QMenu(self.parent)
        self.setContextMenu(self.menu)

        self.show()

    def addAction(self, action:QAction):
        if type(action) != QAction:
            print('添加的Action不是一个QAction对象。')
            return

        self.menu.addAction(action)

    def __del__(self):
        self.hide()


from qqApi import qqApi

from configNeteaseFeatures import ConfigNetEase


class ConfigQQ(ConfigNetEase):

    def __init__(self, parent):
        super(ConfigQQ, self).__init__(parent)

        self.api = qqApi
    
        self.ein = 29
    
    def getSings(self):
        result = self.api.playList(ein=self.ein)
        if not result:
            return 
            
        for i in result:
            self.result.append(i)
            self.singNames.append(i['dissname'])
            self.singPicUrls.append(i['imgurl'])
            self.playlistIds.append(int(i['dissid']))

        self.ein += 30
    
    def requestsDetail(self, ids):
        reqResult = self.api.getPlaylist(ids)
        self.reqResult = reqResult
        if not self.reqResult:
            print('QQ歌单获取失败，请重新获取。')
            return 
        self.singsIds = [i['songid'] for i in reqResult['tracks']]

        self.singsUrls = [i['mp3Url'] for i in reqResult['tracks']]



__author__ = 'cyrbuzz'

import os
import glob
import os.path

try:
    import eyed3
except ImportError:
    print('eyed3没有成功加载或安装，请不要使用本地音乐功能！')

from base import QFileDialog, QObject, QTableWidgetItem
from addition import itv2time


def getAllFolder(topFolder):
    result = []

    def findFolder(topFolder):
        folders = [os.path.join(topFolder, i) for i in os.listdir(topFolder) if not os.path.isfile(os.path.join(topFolder, i))]
        if not folders:
            return 
        else:
            result.extend(folders)
            for i in folders:
                findFolder(i)

    findFolder(topFolder)

    return result


class ConfigNative(QObject):

    def __init__(self, native):
        super(ConfigNative, self).__init__()
        self.native = native

        self.musicList = []
        self.folder = []

        self.bindConnect()

    def bindConnect(self):
        self.native.selectButton.clicked.connect(self.selectFolder)
        self.native.singsTable.itemDoubleClicked.connect(self.itemDoubleClickedEvent)

    def selectFolder(self):
        folder = QFileDialog()
        selectFolder = folder.getExistingDirectory()
        if not selectFolder:
            pass
        else:
            self.folder.append(selectFolder)

            mediaFiles = glob.glob(selectFolder+'/*.mp3')
            allFolder = getAllFolder(selectFolder)
            for i in allFolder:
                mediaFiles.extend(glob.glob(i+'/*.mp3'))

            length = len(mediaFiles)
            self.native.singsTable.clearContents()
            self.native.singsTable.setRowCount(length)
            self.musicList = []
            for i in enumerate(mediaFiles):
                music = eyed3.load(i[1])

                if not music:
                    self.singsTable.removeRow(i[0])
                    continue

                try:
                    name = music.tag.title
                    author = music.tag.artist

                    if not name:
                        filePath = i[1].replace(selectFolder, '')
                        name = filePath[1:][:-4]
                    if not author:
                        author = ''  
                except:
                    try:
                        # TODO
                        # if more folders exist.
                        filePath = i[1].replace(selectFolder, '')
                        name = filePath[1:][:-4]
                    except Exception as e:
                        name = i[1]
                        author = ''
                try:
                    time = itv2time(music.info.time_secs)
                except:
                    time = '00:00'

                self.musicList.append({'name': name, 'author': author, 'time': time, 'url': i[1], 'music_img': 'None'})
                self.native.singsTable.setItem(i[0], 0, QTableWidgetItem(name))
                self.native.singsTable.setItem(i[0], 1, QTableWidgetItem(author))
                self.native.singsTable.setItem(i[0], 2, QTableWidgetItem(time))

    # 事件。
    def itemDoubleClickedEvent(self):
        currentRow = self.native.singsTable.currentRow()
        data = self.musicList[currentRow]

        self.native.parent.playWidgets.setPlayerAndPlayList(data)


__author__ = 'cyrbuzz'
"""制作登陆区盒子。"""

from base import QDialog, QFrame, HBoxLayout, HStretchBox, QLabel, QLineEdit, QPushButton, Qt, VBoxLayout, RequestThread


class InputLine(QLineEdit):

    def __init__(self, parent=None, width=0, height=0, placeholderText=None):
        super(QLineEdit, self).__init__()
        self.parent = parent

        if width:
            self.setMaximumWidth(width)
            self.setMinimumWidth(width)
        
        if height:
            self.setMaximumHeight(height)
            self.setMinimumHeight(height)

        if placeholderText:
            self.setPlaceholderText(placeholderText)


class Header(QFrame):
    myStyle = """
    QFrame {background: #2D2D2D;}

    QLabel {
        margin-left: 8px; 
        color: white; 
        font-weight: bold;
        font-size: 15px;
    }

    QPushButton {
        border: none;
        font: bold;
        font-size: 13px;
        color: #7C7C7C;
        margin-right: 8px;
    }

    QPushButton:hover{
        color: #DCDDE4;
    }
    """

    def __init__(self, title:str, parent=None):
        super(Header, self).__init__()
        self.parent = None

        self.setStyleSheet(self.myStyle)

        self.mainLayout = HBoxLayout(self)
        
        self.title = QLabel(title)
        self.mainLayout.addWidget(self.title)

        self.mainLayout.addStretch(1)

        self.closeButton = QPushButton('×')
        self.mainLayout.addWidget(self.closeButton)

    def connectCloseButton(self, functionName):
        self.closeButton.clicked.connect(functionName)


class LoginBox(QDialog):

    def __init__(self, parent=None):
        super(LoginBox, self).__init__()
        self.parent = parent
        
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setWindowTitle('登陆')
        self.setObjectName('LoginBox')

        self.resize(520, 300)

        # 可能会有多个渠道登陆的后续扩展。
        self.currentFrame = 0

        self.mainLayout = VBoxLayout(self)

        self.phoneAndEMailFrame = PhoneAndEMailFrame(self)

        self.mainLayout.addWidget(self.phoneAndEMailFrame)

    def setWarningAndShowIt(self, warningStr):
        if not self.currentFrame:
            self.phoneAndEMailFrame.setWarningAndShowIt(warningStr)

    def connectLogin(self, functionName):
        if not self.currentFrame:
            self.phoneAndEMailFrame.connectLogin(functionName)
    
    def checkAndGetLoginInformation(self):
        if not self.currentFrame:
            return self.phoneAndEMailFrame.checkAndGetLoginInformation()            


class PhoneAndEMailFrame(QFrame):

    def __init__(self, parent=None):
        super(PhoneAndEMailFrame, self).__init__()
        self.parent = parent
        self.resize(520, 300)
        with open('QSS/phoneAndEMailFrame.qss', 'r') as f:

            self.setStyleSheet(f.read())

        self.mainLayout = VBoxLayout(self)

        self.header = Header("用户名", self)
        self.header.setMinimumHeight(40)
        self.header.connectCloseButton(self.parent.accept)
        self.mainLayout.addWidget(self.header)

        self.mainLayout.addStretch(1)

        self.usernameLine = InputLine(self, 220, 32, '请输入用户名')
        self.usernameLine.setObjectName('usernameLine')
        self.usernameCenterBox = HStretchBox(self.mainLayout, self.usernameLine)   
        
        self.mainLayout.addSpacing(10) 
        
        self.passwordLine = InputLine(self, 220, 32, '请输入密码')
        self.passwordLine.setObjectName('passwordLine')
        self.passwordCenterBox = HStretchBox(self.mainLayout, self.passwordLine)
        self.passwordLine.setEchoMode(QLineEdit.Password)

        self.warningIconLabel = QLabel()
        self.warningIconLabel.setObjectName('warningIconLabel')
        self.warningIconLabel.setMaximumSize(14, 14)
        self.warningIconLabel.setMinimumSize(14, 14)
        self.warningIconLabel.hide()
        self.warningLabel = QLabel("请输入用户名")
        self.warningLabel.hide()
        self.warningLabel.setObjectName('warningLabel')

        self.warningCenterBox = HStretchBox(self.mainLayout, self.warningIconLabel, self.warningLabel, 
            behindStretch=2)

        self.mainLayout.addSpacing(30) 

        self.enterLoginButton = QPushButton("登 录")
        self.enterLoginButton.setObjectName("enterButton")
        self.enterLoginButton.setMaximumSize(217, 27)
        self.enterLoginButton.setMinimumSize(217, 27)
        self.enterLoginCenterBox = HStretchBox(self.mainLayout, self.enterLoginButton)

        self.mainLayout.addSpacing(30)
        
        self.mainLayout.addStretch(1)

    def checkAndGetLoginInformation(self):
        username = self.usernameLine.text()
        password = self.passwordLine.text()
        
        if not username or not password:
            self.warningIconLabel.show()

        if not username:
            self.warningLabel.setText('请输入用户名')
            self.warningLabel.show()
            return False

        if not password:
            self.warningLabel.setText('请输入密码')
            self.warningLabel.show()
            return False

        self.warningIconLabel.hide()
        self.warningLabel.hide()

        return username, password

    def setWarningAndShowIt(self, warningStr):
        self.warningLabel.setText(warningStr)

        self.warningLabel.show()
        self.warningIconLabel.show()

    def connectLogin(self, functionName):
        self.enterLoginButton.clicked.connect(functionName)


if __name__ == '__main__':
    import os
    
    os.chdir('..')

    app = QApplication([])

    main = LoginBox()
    main.show()

    app.exec_()


# 虾米等其他的扩展与netease几乎没有区别，只是api的区别。
# 暂时让netease当基类。
from xiamiApi import xiami

from configNeteaseFeatures import ConfigNetEase


class ConfigXiami(ConfigNetEase):

    def __init__(self, parent):
        super(ConfigXiami, self).__init__(parent)

        self.api = xiami
    
        self.page = 1
    
    def getSings(self):
        result = self.api.playList(page=self.page)
        if not result:
            return False
            
        for i in result:
            self.result.append(i)
            self.singNames.append(i['collect_name'])
            self.singPicUrls.append(i['logo'])
            self.playlistIds.append(i['list_id'])
    
        self.page +=1
    
    def requestsDetail(self, ids):
        reqResult = self.api.getPlaylist(ids)

        self.reqResult = reqResult

        self.singsIds = [i['song_id'] for i in reqResult['tracks']]

        self.singsUrls = [i['listen_file'] for i in reqResult['tracks']]


"""包含音乐的组件, 显示用信息, 一个自定义可HOVER整行效果的TableWidget。"""
"""
这是一个独立的播放器组件，只需要调用playWidgets里的setPlayerAndPlayList(s)即可播放音乐。
需要的data类型是{'name': str, 'time': str, 'author': str, music_img: None/url}
"""
"""
写法上这个模块比较难懂，界面与功能混合在一块。
只要在需要播放音乐时调用
setPlayerAndPlayList(s)设置音乐即可。
必要时会进行重写。
"""
__author__ = 'cyrbuzz'

import re
import os
import sys
import random
import logging

from PyQt5.QtWidgets import (QAction, QAbstractItemView, QDialog, QFrame, QHBoxLayout, QLabel, QMenu, QPushButton, QSlider, 
                                                              QTableWidget, QTableWidgetItem, QVBoxLayout, QApplication)
from PyQt5.QtGui import QBrush, QColor, QCursor, QPixmap, QFont, QPainter, QPen, QLinearGradient
from PyQt5.QtCore import QUrl, Qt, QObject, QPropertyAnimation, QPoint, QRect, QEasingCurve, QAbstractAnimation, QTime, QTimer, QRegExp, QRectF
from PyQt5.QtMultimedia import QMediaPlayer, QMediaContent, QMediaMetaData, QMediaPlaylist

import addition
from base import (blur, checkFolder, cacheFolder, checkOneFolder, centerHTML, HBoxLayout, HStretchBox, 
                                     pickle, PicLabel, pyqtSignal,  QTextEdit,  ScrollArea, VBoxLayout)
# ../features
from asyncBase import aAsync, toTask
# ../apis
from netEaseApi import netease
from xiamiApi import xiami
from qqApi import qqApi

# support desktop lyric.
from desktopLyricButtons import *

def _fromUtf8(s):
    return s

# for desktop lyric too.
QString = str

logger = logging.getLogger(__name__)


# 底部的播放组件。主要是用于交互，包括播放/前进/后退/进度条/音量控制/播放模式/打开or关闭音乐列表。
class PlayWidgets(QFrame):
    """播放组件，包括播放按钮，进度条君，音量君，当前歌曲菜单君。"""

    def __init__(self, parent=None):
        super(PlayWidgets, self).__init__()
        self.setObjectName('playwidget')
        self.parent = parent
        # 0模式是列表循环。1模式是单曲循环，2模式是随机播放。
        self.module = 0

        with open('QSS/playWidgets.qss', 'r', encoding='utf-8') as f:
            self.setStyleSheet(f.read())

        self.playList = PlayList(self)
        self.currentMusic = CurrentMusic(self)
        self.player = Player(self)

        # test desktopLyric
        self.desktopLyric = DesktopLyric(self)

        self.setButtons()
        self.setLabels()
        self.setSliders()

        self.setLayouts()

        self.loadCookies()

    # 布局。
    def setButtons(self):
        """设置所有的按钮组件，包括前/后一首，暂停/播放等。"""
        self.previousButton = QPushButton(self)
        self.previousButton.setObjectName("previousButton")
        self.previousButton.clicked.connect(self.previousSing)

        self.playButton = QPushButton(self)
        self.playButton.setObjectName("playButton")
        self.playButton.clicked.connect(lambda: self.playEvent(self.player))

        self.pauseButton = QPushButton(self)
        self.pauseButton.setObjectName("pauseButton")
        self.pauseButton.clicked.connect(lambda: self.pauseEvent(self.player))

        # 默认hide.
        self.pauseButton.hide()

        self.nextButton = QPushButton(self)
        self.nextButton.setObjectName("nextButton")
        self.nextButton.clicked.connect(self.nextSing)

        self.volume = QPushButton(self)
        self.volume.setObjectName("volume")
        self.volume.clicked.connect(self.volumeEvent)

        self.noVolume = QPushButton(self)
        self.noVolume.setObjectName("no_volume")
        self.noVolume.hide()
        self.noVolume.clicked.connect(self.noVolumeEvent)

        self.single = QPushButton(self)
        self.single.setObjectName("single")
        self.single.hide()
        self.single.setToolTip("单曲循环")
        self.single.clicked.connect(self.singleEvent)

        self.repeat = QPushButton(self)
        self.repeat.setObjectName("repeat")
        self.repeat.setToolTip("列表循环")
        self.repeat.clicked.connect(self.repeatEvent)

        self.shuffle = QPushButton(self)
        self.shuffle.setObjectName("shuffle")
        self.shuffle.hide()
        self.shuffle.setToolTip("随机播放")
        self.shuffle.clicked.connect(self.shuffleEvent)

        self.lyricButton = QPushButton(self)
        self.lyricButton.setText("词")
        self.lyricButton.setToolTip("打开歌词")
        self.lyricButton.setObjectName("lyricButton")
        self.lyricButton.clicked.connect(self.lyricEvent)

        self.playlist = QPushButton(self)
        self.playlist.setObjectName("playlist")
        self.playlist.clicked.connect(self.playlistEvent)

    def setLabels(self):
        """包括两个时间。当前播放时间/歌曲总时间。"""

        self.currentTime = QLabel(self)
        self.countTime = QLabel(self)

        self.currentTime.setText('01:09')
        self.countTime.setText('03:04')

    def setSliders(self):
        """设置进度条君。"""
        self.slider = QSlider(self)
        self.slider.setMinimumHeight(5)
        self.slider.setMinimumWidth(440)
        # 将范围设置成1000滚动时更舒服。
        self.slider.setRange(0, 1000)
        self.slider.setObjectName("slider")
        self.slider.setOrientation(Qt.Horizontal)
        self.slider.sliderReleased.connect(self.sliderEvent)
        self.slider.sliderPressed.connect(self.sliderPressEvent)

        self.volumeSlider = QSlider(self)
        self.volumeSlider.setValue(100)
        self.volumeSlider.setMinimumHeight(5)
        self.volumeSlider.setObjectName("volumeSlider")
        self.volumeSlider.setOrientation(Qt.Horizontal)
        self.volumeSlider.valueChanged.connect(self.volumeChangedEvent)

    def setLayouts(self):
        """设置布局。"""
        self.mainLayout = QHBoxLayout()

        self.mainLayout.addWidget(self.previousButton)
        self.mainLayout.addWidget(self.playButton)
        self.mainLayout.addWidget(self.pauseButton)
        self.mainLayout.addWidget(self.nextButton)

        self.mainLayout.addWidget(self.currentTime)
        self.mainLayout.addWidget(self.slider)
        self.mainLayout.addWidget(self.countTime)

        self.mainLayout.addWidget(self.volume)
        self.mainLayout.addWidget(self.noVolume)

        self.mainLayout.addWidget(self.volumeSlider)
        self.mainLayout.addSpacing(10)

        self.mainLayout.addWidget(self.single)
        self.mainLayout.addWidget(self.repeat)
        self.mainLayout.addWidget(self.shuffle)

        self.mainLayout.addSpacing(10)
        self.mainLayout.addWidget(self.lyricButton)
        self.mainLayout.addSpacing(10)

        self.mainLayout.addWidget(self.playlist)
        self.mainLayout.addStretch(1)
        self.mainLayout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.mainLayout)

    # 功能。
    def previousSing(self):
        # 前一首。
        self.player.playList.previous()

    def nextSing(self):
        # 下一首。
        self.player.playList.next()

    def setPlayerAndPlayList(self, data:dict, index=0):
        """方便外部调用。一键添加歌曲。"""
        """防止重复。"""
        logger.info("尝试添加歌曲到播放列表，歌曲信息: {0} \n".format(data))
        authorAndName = data['author'] + data['name']
        for i, mediaInfo in enumerate(self.playList.musicList):
            checkAuthorAndName = mediaInfo['author'] + mediaInfo['name']
            # if data in self.playList.musicList:
            if authorAndName == checkAuthorAndName:
                # index = self.playList.musicList.index(data)
                if self.player.playList.currentIndex == i and i != 0:
                    return

                self.player.setIndex(i)

                return

        # 添加资源当前项到播放列表。
        # if 'qq.com' in data['url']:
            # print(data['url'])
            # return
            
        sureSetUp = self.player.setMusic(data['url'], data)
        if not sureSetUp:
            self.currentMusic.setShortInfo('音乐不可播放', data['author'], data['music_img'])
            return 
            
        self.playList.addMusic(data)

        self.player.playList.tabMusicEvent()
        # 添加显示播放列表的显示项。
        self.playList.addPlayList(data['name'], data['author'], data['time'])
        # # 更改当前音乐的信息。
        # self.currentMusic.setShortInfo(data['name'], data['author'], data['music_img'])
        # 添加歌曲到播放器。
        # index = len(self.playList.musicList)
        # index = self.player.playList.mediaCount()
        # self.player.setIndex(index-1)

    def setPlayerAndPlaylists(self, datas:list):
        datas = datas[::-1]
        for i in datas:
            # self.playList.addMusic(i)
            self.playList.addPlayList(i['name'], i['author'], i['time'])

        self.playList.addMusics(datas)
        
        self.player.setAllMusics(datas)

        sureSetUp = self.player.setMusic(i['url'], i)
        if not sureSetUp:
            self.currentMusic.setShortInfo('音乐不可播放', data['author'], data['music_img'])
            self.nextSing()
            return
        self.player.playList.tabMusicEvent()

        # self.currentMusic.setShortInfo(i['name'], i['author'], i['music_img'])

    def saveCookies(self):
        self.playList.saveCookies()
        self.player.saveCookies()

    def loadCookies(self):
        self.playList.loadCookies()
        self.player.loadCookies()

    # 事件。
    def playEvent(self, media):
        """播放事件。"""
        self.playButton.hide()
        self.pauseButton.show()
        media.play()

    def pauseEvent(self, media):
        """暂停事件。"""
        self.pauseButton.hide()
        self.playButton.show()
        media.pause()

    def stopEvent(self, media):
        self.pauseButton.hide()
        self.playButton.show()
        media.stop()

    def volumeEvent(self):
        """有音量时事件。"""
        self.volume.hide()
        self.noVolume.show()
        self.volumeSlider.setValue(0)

    def noVolumeEvent(self):
        """静音后事件。"""
        self.noVolume.hide()
        self.volume.show()
        self.volumeSlider.setValue(100)

    def volumeChangedEvent(self):
        """音量改变事件。"""
        self.player.setVolume(self.volumeSlider.value())

    def singleEvent(self):
        """单曲循环事件。将模式更改为随机播放。"""
        self.single.hide()
        self.shuffle.show()
        # 设置成随机。详情看http://doc.qt.io/qt-5/qmediaplaylist.html#PlaybackMode-enum。
        self.player.playList.setPlaybackMode(4)
        self.module = 2

    def repeatEvent(self):
        """列表循环事件。将模式更改为单曲循环。"""
        self.repeat.hide()
        self.single.show()
        self.player.playList.setPlaybackMode(1)
        self.module = 1

    def shuffleEvent(self):
        """随机播放事件。将模式更改为列表循环。"""
        self.shuffle.hide()
        self.repeat.show()
        self.player.playList.setPlaybackMode(3) 
        self.module = 0

    def lyricEvent(self):
        """return None"""
        lyricStyle = """
                QPushButton#lyricButton {{
                    width: 10px;
                    height: 10px;
                    margin: 0px;
                    padding: 3px;
                    background: {};
                    color: {};
                    border: 1px solid #919192;
                }}
            QPushButton#lyricButton:hover {{
                background: none;
                color: #DCDCDC;
                border-color: #DCDCDC;
            }}
                """
        # 如果桌面歌词处于可见状态则隐藏桌面歌词。
        if self.desktopLyric.isVisible():
            self.lyricButton.setToolTip("打开歌词")
            self.lyricButton.setStyleSheet(lyricStyle.format("none", "#79797B"))
            return self.desktopLyric.hide()

        # 防止先打开的音乐后显示的歌词，所以在点击时获取一次歌词。
        self.lyricButton.setToolTip("关闭歌词")
        self.lyricButton.setStyleSheet(lyricStyle.format("#828282", "#FFFFFF"))
        self.desktopLyric.show()
        self.currentMusic.setDetailInfo()

    def playlistEvent(self):
        """用于显示播放列表或隐藏播放列表。"""
        if self.playList.isVisible():
            self.playList.hide()
        else:
            self.playList.move(self.parent.width()-574, self.parent.height()-477-self.height())
            self.playList.show()
            self.playList.raise_()

    def sliderEvent(self):
        """拖动进度条的事件，用于快进快退。"""
        value = self.slider.value()
        currentSliderTime = self.player.allTime()*value
        self.player.setPosition(currentSliderTime)

        # 按下时将会取消进度条的自动更新，现在恢复它。
        self.player.setConnects()
        # 将进度条移动的事件清除，因为自动更新是不需要的。
        self.slider.sliderMoved.disconnect()

    def sliderPressEvent(self):
        """按下准备拖动时。"""
        # 将进度条自动更新取消。
        self.player.setDisconnects()
        # 添加进度条移动事件。
        self.slider.sliderMoved.connect(self.sliderMovedEvent)

    def sliderMovedEvent(self):
        """用于拖动时显示当前位置的时间。"""
        # 进度条移动事件，会显示当前的时间。
        value = self.slider.value()
        currentSliderTime = self.player.allTime()*value/1000
        currentSliderTime = addition.itv2time(currentSliderTime)
        self.currentTime.setText(currentSliderTime)
        self.player.timeChanged.emit(currentSliderTime)


# 显示用音乐列表，用于显示当前添加到播放列表里的音乐。
class PlayList(QFrame):
    """播放列表。"""
    musicListCookiesFolder = 'cookies/playlist/musicList.cks'
    allCookiesFolder = [musicListCookiesFolder]

    def __init__(self, parent=None):
        super(PlayList, self).__init__()
        
        self.parent = parent
        
        self.setParent(self.parent.parent)
        self.setObjectName("PlayList")

        self.musicList = []

        self.currentRow = -1

        self.allRow = 0

        self.itemColor = QBrush(QColor.fromRgb(95,95,99))

        with open('QSS/playList.qss', 'r') as f:
            self.setStyleSheet(f.read())

        self.resize(574, 477)

        self.hide()

        self.setButtons()
        self.setLabels()
        self.setTables()

        self.setLayouts()

    # 布局。
    def setButtons(self):
        self.closeButton = QPushButton("×", self)
        self.closeButton.setObjectName("closeButton")
        self.closeButton.clicked.connect(self.hide)

    def setLabels(self):
        pass

    def setTables(self):
        self.playList = _TableWidget(self)
        self.playList.setMinimumWidth(self.width())
        self.playList.setColumnCount(3)
        # 显示名字的部分占全部长度的2/3.
        self.playList.setColumnWidth(0, self.width()/3*2)
        self.playList.horizontalHeader().setVisible(False)
        self.playList.horizontalHeader().setStretchLastSection(True)
        self.playList.verticalHeader().setVisible(False)
        self.playList.setShowGrid(False)
        self.playList.setAlternatingRowColors(True)

        self.playList.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.playList.setSelectionBehavior(QAbstractItemView.SelectRows)

        self.playList.itemDoubleClicked.connect(self.play)

    def setLayouts(self):
        self.mainLayout = QVBoxLayout()

        self.headerLayout = QHBoxLayout()
        self.headerLayout.addStretch(1)
        self.headerLayout.addWidget(self.closeButton)

        self.mainLayout.addLayout(self.headerLayout)
        self.mainLayout.addWidget(self.playList)

        self.setLayout(self.mainLayout)

    # 功能。
    def play(self):
        """将当前选中的歌曲添加到播放列表。"""
        if self.currentRow == self.playList.currentRow():
            return
        else:
            # 以MediaContent为索引，是一个字典，包括url，name, author。
            currentRow = self.playList.currentRow()
            
            logger.info("尝试播放已在播放列表的歌曲，歌曲位置索引{0}.".format(currentRow))

            currentMusic = self.musicList[currentRow]

            self.parent.player.setIndex(currentRow)
            
            self.parent.currentMusic.setShortInfo(currentMusic['name'], currentMusic['author'])

            self.currentRow = currentRow

    def clear(self):
        self.playList.clears()

    def addMusic(self, data):
        self.musicList.append(data)

    def addMusics(self, mlist):
        self.musicList.extend(mlist)

    def addPlayList(self, name, author, time):
        # 将信息添加到显示列表。
        self.playList.setRowCount(self.allRow+1)

        musicName = QTableWidgetItem(name)
        self.playList.setItem(self.allRow, 0, musicName)

        musicAuthor = QTableWidgetItem(author)
        musicAuthor.setForeground(self.itemColor)
        self.playList.setItem(self.allRow, 1, musicAuthor)

        musicTime = QTableWidgetItem(time)
        musicTime.setForeground(self.itemColor)
        self.playList.setItem(self.allRow, 2, musicTime)

        self.allRow += 1

    @checkFolder(allCookiesFolder)
    def saveCookies(self):
        with open(self.musicListCookiesFolder, 'wb') as f:
            pickle.dump(self.musicList, f)

    @checkFolder(allCookiesFolder)
    def loadCookies(self):
        with open(self.musicListCookiesFolder, 'rb') as f:
            self.musicList = pickle.load(f)

        for i in self.musicList:
            self.addPlayList(i['name'], i['author'], i['time'])


# 当前播放的音乐，位于左侧导航栏最下方，播放组件的上方。包括当前音乐的logo，音乐名/作者。点击图片会显示详细信息(暂不完善。)
class CurrentMusic(QFrame):
    """当前正在播放的音乐，包括一个图片，音乐名称，作曲人。以及鼠标移动到上面的遮罩。"""

    def __init__(self, parent=None):
        super(CurrentMusic, self).__init__()
        self.parent = parent
        self.grandparent = self.parent.parent
        self.setParent(self.grandparent)
        self.setObjectName("currentMusic")

        with open('QSS/currentMusic.qss', 'r') as f:
            self.setStyleSheet(f.read())

        # 用于标记是否切换了歌曲，防止多次获取同一个歌词。
        self.currentMusicId = 0
            
        # 用于切换歌词，知道当前的歌词滚到哪一个了。
        self.order = -2

        # 用于切换歌词时滚动滚动条。
        # self.sliderValue = 0

        # 每一条歌词该滚多少。
        # self.slideValue = 0

        # 有多少条歌词控件。
        self.count = 0

        # 歌词缓存。
        self.lyricCache = ''

        # 将窗口提前并激活窗口。
        self.raise_()
        self.activateWindow()

        self.shortInfo = CurrentMusicShort(self)
        self.detailInfo = CurrentMusicDetail(self)
        
        # 
        self.showDetail = 'animation'
        self.showShort = 'animation'
        self.mousePos = None

        self.mainLayout = QHBoxLayout(self)
        self.mainLayout.addWidget(self.shortInfo)
        self.mainLayout.addWidget(self.detailInfo)
        # 保证没有间隙。
        self.mainLayout.setContentsMargins(0, 0, 0, 0)
        self.mainLayout.setSpacing(0)

        self.setFeatures()

    # 功能。
    @checkOneFolder(cacheFolder)
    def setShortInfo(self, name=None, author=None, pic=None):
        """方便设置信息。"""
        if pic:
            cacheList = os.listdir(cacheFolder)
            names = str(pic[pic.rfind('/')+1:])
            if names in cacheList:
                pic = cacheFolder+'/'+names

            self.shortInfo.musicPic.setSrc(pic)

        self.shortInfo.musicName.setText(name)
        self.shortInfo.musicAuthor.setText(author)

    def setDetailInfo(self):
        title = self.shortInfo.musicName.text()
        
        picSrc = self.shortInfo.musicPic.getSrc()
        
        blurSrc = blur(picSrc)

        self.detailInfo.titleLabel.setText(title)
        if blurSrc:
            self.detailInfo.lyricFrames.setStyleSheet('''
            QScrollArea#lyricScroll {{
                background-image: url({0});
            }}
                '''.format(blurSrc))

        self.showLyric()

    def getDetailInfo(self):
        """点击后进行动画效果的: 显示某歌曲的详细信息。"""
        self.shortInfo.hide()
        self.detailInfo.show()

        self.showDetail = QPropertyAnimation(self, b"geometry")

        x = self.pos().x()
        y = self.pos().y()

        self.showDetail.setStartValue(QRect(x, y, self.width(), self.height()))
        # 获取顶层父窗口的长度。
        self.showDetail.setEndValue(QRect(0, self.grandparent.header.height()+3, self.grandparent.width(), self.grandparent.mainContent.height()))
        self.showDetail.setDuration(300)
        self.showDetail.setEasingCurve(QEasingCurve.InBack)

        self.showDetail.start(QAbstractAnimation.DeleteWhenStopped)
        # 将该组件显示在最前，默认会嵌入到父组件里面。
        self.raise_()

        self.setDetailInfo()

    def showLyric(self):
        lyric = self.getLyric()
        # lyric.add_done_callback(lambda future: self.detailInfo.detailText.setText(centerHTML(future.result)()))
        lyric.add_done_callback(self.lyricCallback)

    def lyricCallback(self, future):
        """lyric加载完成后的回调函数。"""
        # [00:00.00] 作曲 : cyr
        # [00:01.00] 作词 : buzz
        # [00:17.460]
        # [00:33.630] :)

        self.detailInfo.removeAllLyricLabels()
        # 初始化。
        self.count = 0
        self.order = -2
        
        result = future.result().split('\n')
        
        # set desktop lyric list.
        self.parent.player.lrc_lst = getLyric(result)

        signal = self.parent.player.timeChanged
        
        for x, i in enumerate(result):
            data = re.findall(r'\[+(.*?)\]+(.*)', i)
            # [''] or [':)']
            if not data:
                if not i:
                    continue
                else:
                    self.detailInfo.addLyricLabel(_LyricLabel('00:00', i,  x, signal, self))
                    continue

            # [00:17.460]
            if not data[0][1]:
                self.detailInfo.addLyricLabel(_LyricLabel(data[0][0], '\n',  x, signal, self))
                continue

            # [00:01.00] 作词 : buzz
            self.detailInfo.addLyricLabel(_LyricLabel(data[0][0], data[0][1],  x, signal, self))
        
        self.count = x

        # 这边并不会返回添加了控件后的Value值。
        # self.sliderValue = self.detailInfo.maximumValue()
        # self.slideValue = round(self.sliderValue/x)

    def  slide(self):
        # 待优化。
        # 问题如上所说。
        # value = self.detailInfo.verticalScrollBar().value()
        maxValue = round(self.detailInfo.lyricFrames.maximumValue()/self.count)*self.order
        # for i in range(value, maxValue):
        self.detailInfo.lyricFrames.verticalScrollBar().setValue(maxValue)

    def unLightLyric(self):
        if self.order < 0:
            return 

        self.detailInfo.allLyrics[self.order].unLightMe()

    @toTask
    def getLyric(self):
        musicInfo = self.parent.player.getCurrentMusicInfo()
        logger.info("尝试获取歌曲歌词地址。 当前歌曲信息: {0}".format(musicInfo))
        if not musicInfo:
            return "✧请慢慢欣赏~"

        if 'http' not in musicInfo.get('url'):
            future = aAsync(xiami.search, musicInfo.get('name') + ' ' + musicInfo.get('author'))
            data = yield from future
            # 最好做下判断以免搜不到获取时导致报错退出。
            lyricUrl = data['songs'][0].get('lyric')
            future = aAsync(xiami.lyric, lyricUrl)
            data = yield from future

            self.lyricCache = data

            return data

        musicId = musicInfo.get('music_id')
        if self.currentMusicId == musicId:
            return self.lyricCache
            # return self.detailInfo.detailText.toPlainText()

        lyricUrl = musicInfo.get('lyric')
        # 默认网易云，网易云没有返回歌词地址，所以会是None.
        if not lyricUrl:
            future = aAsync(netease.lyric, musicId)
            data = yield from future
        else:
            if 'xiami' in lyricUrl:
                future = aAsync(xiami.lyric, lyricUrl)
                data = yield from future
            elif lyricUrl == 'qq':
                # TODO
                # get qq music lyric.
                return "✧请慢慢欣赏~"


        if not data:
            self.currentMusicId = musicId
            return "✧请慢慢欣赏~"

        # 这里暂时处理下不获取时间信息，只获取歌词信息。
        # data = re.sub(r'\[.*?\]', '', data)
        self.currentMusicId = musicId
        self.lyricCache = data
        return data    

    def getShortInfo(self):
        """返回到原来的缩略图信息。"""
        self.detailInfo.hide()
        self.showShort = QPropertyAnimation(self, b"geometry")
        
        x = self.pos().x()
        y = self.pos().y()
        
        self.showShort.setStartValue(QRect(0, self.grandparent.header.height(), self.grandparent.width(), self.grandparent.mainContent.height()))
        self.showShort.setEndValue(QRect(0, self.grandparent.height()-64-self.parent.height(), self.grandparent.navigation.width(), 64))
        self.showShort.setDuration(300)
        self.showShort.setEasingCurve(QEasingCurve.InBack)
        
        self.showShort.start(QAbstractAnimation.DeleteWhenStopped)

        self.shortInfo.show()
        self.raise_()

    def setFeatures(self):
        self.detailInfo.recoveryButton.clicked.connect(self.getShortInfo)

    # 事件。
    def mousePressEvent(self, event):
        self.mousePos = QCursor.pos()

    def mouseReleaseEvent(self, evnet):
        if QCursor.pos() != self.mousePos:
            return
        else:
            self.getDetailInfo()


# 显示简短信息（默认。）
class CurrentMusicShort(QFrame):
    """用于显示当前歌曲的简单信息。"""
    def __init__(self,parent=None):
        super(CurrentMusicShort, self).__init__()
        self.parent = parent
        self.setObjectName('short')

        self.mousePos = None

        self.setLabels()

        # self.setButtons()

        self.setLayouts()

        self.init()

    # 布局。
    def setLabels(self):
        self.musicName = QLabel(self)
        self.musicName.adjustSize()
        self.musicAuthor = QLabel(self)

        self.musicPic = PicLabel('resource/no_music.png', 64, 64)
        self.musicPic.setObjectName("musicPic")

        self.musicMask = PicLabel('resource/expand.png', 64, 64)
        self.musicMask.hide()
        # 设置背景透明。
        self.musicMask.setStyleSheet('QLabel {background-color: rgba(0, 0, 0, 50%;)}')

        # 遮罩层属于musicPic, 用布局简单弄一下即可。
        self.musicLayout = VBoxLayout(self.musicPic)
        self.musicLayout.addWidget(self.musicMask)

    def setLayouts(self):
        """布局。"""
        self.mainLayout = QHBoxLayout()
        self.mainLayout.setContentsMargins(0, 0, 0, 0)

        self.musicInfo = QVBoxLayout()
        self.mainLayout.setContentsMargins(0, 0, 0, 0)

        self.musicInfo.addWidget(self.musicName)
        self.musicInfo.addWidget(self.musicAuthor)

        self.mainLayout.addWidget(self.musicPic)
        self.mainLayout.addLayout(self.musicInfo)

        self.mainLayout.setStretch(0, 1)
        self.mainLayout.setStretch(1, 2)

        self.setLayout(self.mainLayout)

    # 功能.
    def init(self):
        """默认情况下的显示，包括音乐图片，音乐名和音乐作者。"""
        self.musicName.setText("Enjoy it")
        self.musicAuthor.setText("Enjoy it")

    # 事件。
    def enterEvent(self, event):
        if not self.musicMask.isVisible():
            self.musicMask.show()

    def leaveEvent(self, event):
        if self.musicMask.isVisible():
            self.musicMask.hide()


# 显示详细信息（点击后的信息，不完善。）
class CurrentMusicDetail(ScrollArea):
    """
    showPic | MusicDetails
    ---------------------
    comments.

    MusicDetails
    title| recovery
    T        E        X        T
    """
    def __init__(self, parent):
        super(CurrentMusicDetail, self).__init__()

        self.setObjectName('detail')
        self.hide()

        self.mainLayout = VBoxLayout(self.frame)
        self.topLayout = HBoxLayout()
        self.topMainLayout = VBoxLayout()
        self.topHeaderLayout = HBoxLayout()

        self.lyricFrames = ScrollArea()
        self.lyricFrames.setObjectName('lyricScroll')
        self.lyricFrames.frame.setObjectName('lyricFrame')
        self.lyricFramesLayout = VBoxLayout(self.lyricFrames.frame)
        # self.lyricFrames.setMaximumHeight(500)
        # 为歌词创建索引方便删除。
        self.allLyrics = []

        # self.detailText = QTextEdit()
        # self.detailText.setObjectName('detailText')
        # self.detailText.setReadOnly(True)

        self.titleLabel = QLabel("✧✧✧")
        self.titleLabel.setObjectName('titleLabel')

        self.recoveryButton = QPushButton()
        self.recoveryButton.setObjectName('recoveryButton')
        self.recoveryButton.setMinimumSize(24, 24)
        self.recoveryButton.setMaximumSize(36, 36)

        self.setLayouts()

    def setLayouts(self):
        self.mainLayout.addLayout(self.topLayout)

        # 为showPic预留。
        # self.topLayout.addStretch(1)
        
        self.topLayout.addLayout(self.topMainLayout)
        self.topMainLayout.addSpacing(25)
        self.topMainLayout.addLayout(self.topHeaderLayout)
        self.topHeaderLayout.addStretch(1)
        self.topHeaderLayout.addSpacing(100)
        self.topHeaderLayout.addWidget(self.titleLabel)
        self.topHeaderLayout.addStretch(1)
        self.topHeaderLayout.addSpacing(20)
        self.topHeaderLayout.addWidget(self.recoveryButton)
        self.topHeaderLayout.addSpacing(50)
        self.topMainLayout.addSpacing(30)
        self.topMainLayout.addWidget(self.lyricFrames)

    def addLyricLabel(self, label):

        HStretchBox(self.lyricFramesLayout, label)
        
        self.allLyrics.append(label)

    def removeAllLyricLabels(self):
        for i in self.allLyrics:
            i.deleteLater()
        self.allLyrics = []

        for i in range(self.lyricFramesLayout.count()):
            self.lyricFramesLayout.takeAt(i)


# 真 · 播放器，用于播放音频。
class Player(QMediaPlayer):
    """播放器组件。"""

    timeChanged = pyqtSignal(str)

    def __init__(self, parent=None):
        super(Player, self).__init__()
        self.setObjectName('player')

        self.parent = parent
        self.playWidgets = self.parent
        self.transTime = addition.itv2time
        self.musicTime = 0
        self.playList = _MediaPlaylist(self)
        # 默认列表循环。
        self.playList.setPlaybackMode(self.playList.Loop)

        # lyric from CurrentMusic setting.
        self.lrc_lst = []

        self.setConnects()

    # 功能。
    def setConnects(self):
        """用于设置连接的信号槽。"""
        self.durationChanged.connect(self.countTimeEvent)
        self.positionChanged.connect(self.positionChangedEvent)
        self.stateChanged.connect(self.stateChangedEvent)
        self.error.connect(self.dealError)
        self.playList.setInitConnection()
        # self.currentMediaChanged.connect(self.currentMediaChangedEvent)
        # self.mediaStatusChanged.connect(self.mediaStatusChangedEvent)

    def setDisconnects(self):
        self.durationChanged.disconnect()
        self.positionChanged.disconnect()
        self.stateChanged.disconnect()
        self.error.disconnect()
        self.mediaStatusChanged.disconnect()
        # self.playList.setInitConnection()

    def setMusic(self, url, data):
        """设置当前的音乐，可以直接用网络链接。"""
        if url:
            if 'http' in url or 'file' in url:
                self.playList.addMedias(QMediaContent(QUrl(url)), data)
            else:
                self.playList.addMedias(QMediaContent(QUrl.fromLocalFile(url)), data)
            return True
        return False
    
    def setAllMusics(self, datas):
        self.playList.addAllMedias(datas)

    def setIndex(self, index):
        self.playList.setCurrentIndex(index)

    def allTime(self):
        """返回当前音乐的总时间。（秒）"""
        return self.duration()/1000

    def getCurrentMusicInfo(self):

        return self.playList.mediaList.get(self.currentMedia().canonicalUrl().toString())

    def playMusic(self, url=None, data=None):
        """播放音乐。"""
        if url:
            self.setMusic(url, data)
            # 播放事件。
            self.playWidgets.playEvent(self)
        else:
            self.playWidgets.playEvent(self)
        
    def dealError(self, error):
        # 具体内容看文档:
        # http://doc.qt.io/qt-5/qmediaplayer.html
        if error:
            try:
                musicInfo = self.playList.mediaList.pop(self.currentMedia().canonicalUrl().toString())
                self.loadRealMusicUrl(musicInfo)
            except:
                logger.error("尝试重新获取音乐地址出错。", exc_info=True)
                print('尝试重新获取音乐地址出错，请清空或删除无效的音乐信息后重试。')

    @toTask
    def loadRealMusicUrl(self, musicInfo):
        # 如果有个Url出错，比如音乐地址403重新获取下地址。
        musicId = musicInfo.get('music_id')
        
        logger.info("网易云歌曲地址失效，尝试重新获取，id号: {0}".format(musicId))

        if not musicId:
            self.playWidgets.nextSing()
            return

        future = aAsync(netease.singsUrl, [musicId])
        data = yield from future

        if not data:
            self.playWidgets.nextSing()
            return 

        logger.info("id {0}'s response: {1}".format(musicId, data))
        
        if not data:
            print("获取音乐地址失败，请检查网络后重试。")
            self.playList.mediaList[url] = musicInfo
            return

        url = data[0]['url']
        musicInfo['url'] = url
        self.playList.mediaList[url] = musicInfo
        # replaceIndex = self.playList.musics.index(invalidUrl)

        self.playList.musics[self.playList.currentIndex] = url
        self.playList.play()

    def saveCookies(self):
        self.playList.saveCookies()

    def loadCookies(self):
        self.playList.loadCookies()

    # 事件。
    def countTimeEvent(self, duration):
        """总时间改变的事件。相当于加载完成歌曲的事件。"""
        self.musicTime = duration / 1000

        self.playWidgets.countTime.setText(self.transTime(self.musicTime))
        self.playList.duration = duration

    def positionChangedEvent(self, position):
        """音乐在Media里以毫秒的形式保存，这里是播放时的进度条。"""
        currentTime = self.position()/1000
        transedTime = self.transTime(currentTime)
        self.playWidgets.currentTime.setText(transedTime)
        self.timeChanged.emit(transedTime)

        # position先于duration变化，会出现/0的情况。
        if self.musicTime == 0:
            return
        # *1000是为了与进度条的范围相匹配。
        self.playWidgets.slider.setValue(currentTime/self.musicTime*1000)
        self.setLyricEvent(position)

    def stateChangedEvent(self):
        """"""
        if self.state() == 0 and self.playList.mediaCount() == 0 and self.playWidgets.pauseButton.isVisible():
            self.playWidgets.stopEvent(self)

    def setLyricEvent(self, position):
        # copy from https://github.com/wn0112/PPlayer
        t = QTime(0, 0, 0)
        t = t.addMSecs(int(position))
        lycF = ''
        lycL = ''
        lycM = ''
        if self.lrc_lst:
            lenOfLrc = len(self.lrc_lst)
            for i in range(lenOfLrc):
                if t.toString("mm:ss") in self.lrc_lst[i][0]:                    
                    t1 = t
                    if i < lenOfLrc - 1:
                        x = self.lrc_lst[i+1][0].replace('[', '')
                        x = x.replace(']', '')
                        t1 = QTime().fromString(x, 'mm:ss.z')
                        intervel = t.msecsTo(t1)
                    else:
                        t1 = QTime().fromString('00:10.99')
                        intervel = 3000
                    self.parent.desktopLyric.stopMask()
                    self.parent.desktopLyric.setText(self.lrc_lst[i][1], intervel)                
                    self.parent.desktopLyric.startMask()
                    if i > 0:
                        lycM = self.lrc_lst[i-1][1]
                    j = 0
                    while(j < i-1):
                        lycF += self.lrc_lst[j][1]+'\n'
                        j += 1
                    j = i
                    while(j < lenOfLrc - 1):
                        lycL += self.lrc_lst[j+1][1]+'\n'
                        j += 1
                    # self.parent.desktopLyric.setText(lycF, lycM, self.lrc_lst[i][1], lycL, intervel)
                    # self.parent.desktopLyric.setText(lycF, lycM, self.lrc_lst[i][1], lycL, intervel)

                    break

    # def mediaStatusChangedEvent(self, status):
        """"""
        # 8是无效音频。
        # if status == 8:
            # print(self.playList.currentIndex(), "当前下标")


# 自定义的QTableWidget, 做了hover一整行的操作，和右键菜单。
class _TableWidget(QTableWidget):

    def __init__(self,parent=None):
        super(_TableWidget, self).__init__()
        self.parent = parent

        self.setMouseTracking(True)
        self.entered.connect(self.itemHover)
        self.viewportEntered.connect(self.viewportHover)
        # #2D2F33
        self.itemDualBackground = QColor.fromRgb(45,47,51)
        # #303236
        self.itemBackground = QColor.fromRgb(48,50,54)
        # #323438
        self.itemHoverBackground = QColor.fromRgb(50,52,56)
        # #E2E2E2
        self.itemHoverColor = QColor.fromRgb(226,226,226)
        # #DCDDE4
        self.itemColor = QColor.fromRgb(220,221,228)
        # #5F5F63
        self.itemColor2 = QColor.fromRgb(95,95,99)
        
        # 当前记录的行数，用于判断鼠标是否离开了当前行。
        self.itemHoverRow = -1
        self.items = []

        self.setActions()

    # 功能。
    def setActions(self):
        
        self.actionClear = QAction('清空', self)
        self.actionClear.triggered.connect(self.clears)

        self.actionRemove = QAction('从列表中删除', self)
        self.actionRemove.triggered.connect(self.remove)

    def _setItems(self, row):
        # 用于记录一整行的item，设置这一整行item的foreground, foreground 是前景色但是只有字体颜色变化了。
        # background, 设置无效,不清楚为什么。
        # 并刷新当前记录的行数。

        self.items.append(self.item(row, 0)) 
        self.items.append(self.item(row, 1))
        self.items.append(self.item(row, 2))

        for i in self.items:
            if i:
                i.setForeground(self.itemHoverColor)
                i.setBackground(self.itemHoverBackground)

        self.itemHoverRow = row

    def itemHover(self, rowItem):
        # 为了实现整行的hover效果。
        row = rowItem.row()

        if row != self.itemHoverRow:
            # 执行第一次初始化。
            if not self.items:
                self._setItems(row)
            # 不是第一次，之前有过
            else:
                # 判断奇偶行还原。
                if row % 2 == 0:
                    for i in self.items:
                        i.setBackground(self.itemDualBackground)
                else:
                    for i in self.items:
                        i.setBackground(self.itemBackground)

                # 还原字体的颜色，第一个为color，其他的为color2.
                self.items[0].setForeground(self.itemColor)
                for i in self.items[1:]:
                    i.setForeground(self.itemColor2)

                # 重新记录并为当前的控件附上相应设置。
                self.items = []
                self._setItems(row)

    def viewportHover(self):
        # 用于还原item到初始化状态。
        if not self.items:
            return

        self.items[0].setForeground(self.itemColor)
        for i in self.items[1:]:
            i.setForeground(self.itemColor2)

        self.items = []
        self.itemHoverRow = -1

    def clears(self):
        """执行清除。"""
        # 删除table里的item。
        # clearContents只清除内容不清除行，清除后如果不设置行为0，行会保留，但是没有QTableWidgetItem，
        # setFroeground会出错，因为None对象是不能设置的。
        self.clearContents()
        self.setRowCount(0)
        playlist = self.parent.parent.player.playList
        playlist.clear()
        playlist.removeList = []
        playlist.mediaList = {}
        # 将音乐信息及索引回归初始化。
        self.parent.musicList = []
        self.parent.currentRow = -1
        self.parent.allRow = 0
        self.items = []
        self.itemHoverRow = -1
        # 将播放器停止。
        self.parent.parent.player.stop()

    def remove(self):
        """从播放列表里删除某歌曲。"""
        row = self.currentRow()
        player = self.parent.parent.player
        # 删除table里的item。
        self.removeRow(row)

        # 剔除音乐信息。
        self.parent.musicList = self.parent.musicList[:row] + self.parent.musicList[row+1:]
        player.playList.removeMedias(row)
        # 将总索引-1.
        self.parent.allRow -= 1
        # 如果删除的是当前记录的行数。
        # 还原到初始状态。
        # items用于一些显示效果。
        if self.itemHoverRow == row:
            self.itemHoverRow = -1
            self.items = []

        # 如果当前播放的音乐是要删除的音乐。
        # if self.parent.currentRow == row:
        #     # 停止后会触发stateChangeEvent。
        #     # 如果是当前的音乐会切换到下一首，如果没有了就停止，如果到达列表底部则切回顶部。
        #     self.parent.parent.player.stop()

    # 事件。
    def contextMenuEvent(self, event):
        item = self.itemAt(self.mapFromGlobal(QCursor.pos()))
        self.menu = QMenu(self)

        if not item:
            self.menu.addAction(self.actionClear)
        else:
            self.menu.addAction(self.actionRemove)
            self.menu.addAction(self.actionClear)

        self.menu.exec_(QCursor.pos())


class _MediaPlaylist(QObject):
    musicsCookiesFolder = 'cookies/mediaPlaylist/musics.cks'
    mediaListCookiesFolder = 'cookies/mediaPlaylist/mediaList.cks'
    allCookiesFolder = [musicsCookiesFolder, mediaListCookiesFolder]

    def __init__(self, parent=None):
        super(_MediaPlaylist, self).__init__()
        self.parent = parent
        self.playWidgets = self.parent.parent

        self.duration = 0

        self.musics = []

        self.currentIndex = 0

        # 用于记录歌曲的信息。
        self.mediaList = {}

        # models.
        self.single = 1
        self.Loop = 3
        self.Random = 4

        # currentModel
        # 默认是3.循环。
        self.myModel = 3

    def setInitConnection(self):
        self.parent.mediaStatusChanged.connect(self.mediaStatusChangedEvent)

    def addMedias(self, url, data):
        # url为QMediaContent, data包含这个歌曲的信息。{name, author, url, music_id}
        self.parent.setMedia(url)
        self.musics.append(url)
        self.currentIndex = len(self.musics) - 1
        self.mediaList[url.canonicalUrl().toString()] = data
        self.parent.playMusic()
        
        # window
        self.playWidgets.parent.systemTray.setToolTip('{0}-{1}'.format(data['name'], data['author']))

    def addAllMedias(self, datas):
        # self.musics.extend(datas)
        for i in datas:
            self.musics.append(i['url'])
            self.mediaList[i['url']] = i

    def clear(self):
        self.musics = []
        self.mediaList = {}
        self.currentIndex = 0
        self.duration = 0

    def mediaCount(self):
        return len(self.musics)

    def removeMedias(self, row):

        self.musics.pop(row)

    def next(self):
        if self.myModel == 4:
                index = random.randint(0, len(self.musics)-1)
                self.setCurrentIndex(index)
                return 

        if self.currentIndex + 1 >= len(self.musics):
            self.currentIndex = 0
        else:
            self.currentIndex += 1

        self.play()

    def previous(self):
        if self.myModel == 4:
                index = random.randint(0, len(self.musics)-1)
                self.setCurrentIndex(index)
                return 
                
        if self.currentIndex - 1 <= -1:
            self.currentIndex = 0
        else:
            self.currentIndex -= 1
        
        self.play()

    def play(self):
        try:
            media = self.musics[self.currentIndex]
        except:
            logger.error("unknow error. musics info: {0}".format(self.musics))
            return

        if type(media) == str:
            if 'http' in media or 'file' in media:
                content = QMediaContent(QUrl(media))
            else:
                content = QMediaContent(QUrl.fromLocalFile(media))

            self.musics = self.musics[:self.currentIndex] + [content] + self.musics[self.currentIndex+1:]
            media = content

        self.parent.setMedia(media)
        self.parent.playMusic()

        self.tabMusicEvent()

    def setCurrentIndex(self, index):
        try:
            media = self.musics[index]
        except:
            logger.error("unknow error. musics info: {0}".format(self.musics))
            return

        if type(media) == str:
            if 'http' in media or 'file' in media:
                content = QMediaContent(QUrl(media))
            else:
                content = QMediaContent(QUrl.fromLocalFile(media))

            self.musics = self.musics[:index] + [content] + self.musics[index+1:]
            self.parent.setMedia(content)
        else:
            self.parent.setMedia(media)

        self.parent.playMusic()
        self.tabMusicEvent()

        self.currentIndex = index

    def setPlaybackMode(self, model:int):
        self.myModel = model

    def setShortInfo(self, musicInfo):
        name = musicInfo['name']
        author = musicInfo['author']
        pic = musicInfo['music_img']
        self.playWidgets.currentMusic.setShortInfo(name, author, pic)

    def setLyric(self, musicInfo):
        name = musicInfo['name']
        author = musicInfo['author']
        if self.playWidgets.currentMusic.detailInfo.isVisible() or self.playWidgets.desktopLyric.isVisible():
            self.playWidgets.desktopLyric.setText("{}-{}".format(name, author), 0)
            self.playWidgets.currentMusic.setDetailInfo()

    def setSystemTrayTip(self, tipMessage):
        self.playWidgets.parent.systemTray.setToolTip(tipMessage)

    def updatePlayTimes(self, musicInfo):
        # 网易云歌曲初次播放地址是http(int, int)，这是个错误地址，所以会进行一次重新获取地址，
        # 写的很麻烦，获取成功后会重新设置一次全部信息，这样的话网易云的歌曲就会插入两次，这里尝试只插入一次。
        # 这个方法也不好。
        # 想到的方法是 1. 如果播放地址获取失败会请求一次数据库让其删除一次。
        # TODO:
        # 新方法。
        if 'http(' in musicInfo['url']:
            return

        self.playWidgets.parent.db.addPlayTimesById(musicInfo['music_id'],
            musicInfo['name'],
            musicInfo['author'])

    @checkFolder(allCookiesFolder)
    def saveCookies(self):
        with open(self.musicsCookiesFolder, 'wb') as f:
            for row, data in enumerate(self.musics):
                if type(data) == QMediaContent:
                    url = data.canonicalUrl().toString()
                    self.musics[row] = url

            pickle.dump(self.musics, f)

        with open(self.mediaListCookiesFolder, 'wb') as f:
            pickle.dump(self.mediaList, f)

    @checkFolder(allCookiesFolder)
    def loadCookies(self):
        with open(self.mediaListCookiesFolder, 'rb') as f:
            self.mediaList = pickle.load(f)

        with open(self.musicsCookiesFolder, 'rb') as f:
            self.musics = pickle.load(f)
            for index, url in enumerate(self.musics):
                if '?vkey=' in url:
                    musicInfo = self.mediaList.pop(url)
                    url = url.split('qq.com/')
                    url2 = url[1].split('?vkey=')[0]

                    url = '{0}'.format(qqApi.sip) + url2 + '?vkey={0}'.format(qqApi.key) +\
                               '&guid={0}'.format(qqApi.guid)

                    self.musics[index] = url.format(qqApi.sip, qqApi.key)
                    self.mediaList[url] = musicInfo

    # 事件。
    def mediaStatusChangedEvent(self, status):
        if status == 7:
            # 循环。
            if self.myModel == 3:
                self.next()
            # 随机。
            elif self.myModel == 4:
                index = random.randint(0, len(self.musics)-1)
                self.setCurrentIndex(index)
            # 单曲。
            else:
                self.parent.play()

    def tabMusicEvent(self):
        try:
            indexUrl = self.parent.currentMedia().canonicalUrl().toString()
            musicInfo = self.mediaList[indexUrl]
            self.setShortInfo(musicInfo)

            self.setLyric(musicInfo)

            self.setSystemTrayTip('{0}-{1}'.format(musicInfo['name'], musicInfo['author']))
            self.updatePlayTimes(musicInfo)
        except:
            logger.error("tabbing music error has ignored. index{0} mediaList{1}".format(indexUrl, self.mediaList), exc_info=True)


class _LyricLabel(QLabel):
    """
    显示歌词的Label。
    为Label设置一个时间属性，这样方便歌词滚动。
    """
    __slots__ = ('myTime', 'myLyric', 'myOrder', 'parent')

    def __init__(self, myTime, lyric, myOrder, signal, parent=None):
        super(_LyricLabel, self).__init__(lyric)
        self.setObjectName('lyric')
        self.parent = parent
        self.myTime = myTime[:myTime.rfind('.')]
        self.myLyric = lyric
        self.myOrder = myOrder

        signal.connect(self.lightMe)
        
        self.setMinimumHeight(40)

        # 设置为可复制状态。
        self.setTextInteractionFlags(Qt.TextSelectableByMouse)

    def lightMe(self, currentTime):
        if currentTime == self.myTime:
            if self.parent.order !=  self.myOrder:
                self.parent.unLightLyric()
                self.parent.order = self.myOrder
                self.parent.slide()
                self.setText('<font color="white">{0}</font>'.format(self.myLyric))

    def unLightMe(self):
            self.setText(self.myLyric)
        

class DesktopLyric(QDialog):
    def __init__(self, parent=None):
        super(DesktopLyric, self).__init__()
        self.lyric = QString('Lyric Show.')
        self.intervel = 0
        self.maskRect = QRectF(0, 0, 0, 0)
        self.maskWidth = 0
        self.widthBlock = 0
        self.t = QTimer()
        self.screen = QApplication.desktop().availableGeometry()
        self.setObjectName(_fromUtf8("Dialog"))
        self.setWindowFlags(Qt.CustomizeWindowHint | Qt.FramelessWindowHint | Qt.Dialog | Qt.WindowStaysOnTopHint | Qt.Tool)
        self.setMinimumHeight(65)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.handle = lyric_handle(self)
        self.verticalLayout = QVBoxLayout(self)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.font = QFont(_fromUtf8('微软雅黑, verdana'), 50)
        self.font.setPixelSize(50)
        # QMetaObject.connectSlotsByName(self)
        self.handle.lyricmoved.connect(self.newPos)
        self.t.timeout.connect(self.changeMask)
        
    def changeMask(self):
        self.maskWidth += self.widthBlock
        self.update()
    
    def setText(self, s, intervel=0):
        self.lyric = s
        self.intervel = intervel
        self.maskWidth = 0
        self.update()
        
    def hideLyric(self):
        self.hide()
        # self.emit(SIGNAL('lyrichide()'))

                    
    def leaveEvent(self, event):
        self.handle.leaveEvent(event)
    
    def show(self):

        super(DesktopLyric, self).show()
    
    def hide(self):
        super(DesktopLyric, self).hide()
        self.handle.hide()
        
    def enterEvent(self, event):
        pass
        # self.handle.handler.setFocus()
        # self.handle.show()
    
    def newPos(self, p):
        self.move(self.pos().x() + p.x(), self.pos().y() + p.y())
        # self.move(500, 600)


    def startMask(self):
        self.t.start(100)
            
    def stopMask(self):
        self.t.stop()
        self.update()
        
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setFont(self.font)
        
        linear = QLinearGradient(QPoint(self.rect().topLeft()), QPoint(self.rect().bottomLeft()))
        linear.setStart(0, 10)
        linear.setFinalStop(0, 50)
        linear.setColorAt(0.1, QColor(14, 179, 255));
        linear.setColorAt(0.5, QColor(154, 232, 255));
        linear.setColorAt(0.9, QColor(14, 179, 255));
        
        linear2 = QLinearGradient(QPoint(self.rect().topLeft()), QPoint(self.rect().bottomLeft()))
        linear2.setStart(0, 10)
        linear2.setFinalStop(0, 50)
        linear2.setColorAt(0.1, QColor(222, 54, 4));
        linear2.setColorAt(0.5, QColor(255, 172, 116));
        linear2.setColorAt(0.9, QColor(222, 54, 4));
        
        painter.setPen(QColor(0, 0, 0, 200));
        painter.drawText(QRect(1, 1, self.screen.width(), 60), Qt.AlignHCenter | Qt.AlignVCenter, self.lyric)
        
        painter.setPen(QColor('transparent'));
        self.textRect = painter.drawText(QRect(0, 0, self.screen.width(), 60), Qt.AlignHCenter | Qt.AlignVCenter, self.lyric)

        painter.setPen(QPen(linear, 0))
        painter.drawText(self.textRect, Qt.AlignLeft | Qt.AlignVCenter, self.lyric)
        if self.intervel != 0:
            self.widthBlock = self.textRect.width()/(self.intervel/150.0)
        else:
            self.widthBlock = 0
        self.maskRect = QRectF(self.textRect.x(), self.textRect.y(), self.textRect.width(), self.textRect.height())
        self.maskRect.setWidth(self.maskWidth)
        painter.setPen(QPen(linear2, 0));
        painter.drawText(self.maskRect, Qt.AlignLeft | Qt.AlignVCenter, self.lyric)

    def mousePressEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            self.m_drag = True
            self.m_DragPosition = event.globalPos()-self.pos()
            event.accept()

    def mouseMoveEvent(self, event):
        try:
            if event.buttons() and Qt.LeftButton:
                self.move(event.globalPos()-self.m_DragPosition)
                event.accept()
        except AttributeError:
            pass

    def mouseReleaseEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            self.m_drag = False


class lyric_handle(QDialog):
    # copy from https://github.com/wn0112/PPlayer
    lyricmoved = pyqtSignal(QPoint)
    def __init__(self, parent=None):
        super(lyric_handle, self).__init__(parent)
        self.timer = QTimer()
        self.setObjectName(_fromUtf8("Dialog"))
        self.setWindowFlags(Qt.CustomizeWindowHint | Qt.FramelessWindowHint | Qt.Dialog | Qt.WindowStaysOnTopHint | Qt.Tool)
        self.setStyleSheet('QDialog { background: #2c7ec8; border: 0px solid black;}')
        self.horiLayout = QHBoxLayout(self)
        self.horiLayout.setSpacing(5)
        self.horiLayout.setContentsMargins(0, 0, 0, 0)
        self.horiLayout.setObjectName(_fromUtf8("horiLayout"))
        self.handler = QLabel(self)
        self.handler.setToolTip('Move Lyric')
        self.handler.setPixmap(QPixmap(':/icons/handler.png'))
        self.handler.setMouseTracking(True)
        self.lockBt = PushButton2(self)
        self.lockBt.setToolTip('Unlocked')
        self.lockBt.loadPixmap(QPixmap(':/icons/unlock.png'))
        self.hideBt = PushButton2(self)
        self.hideBt.setToolTip('Hide Lyric')
        self.hideBt.loadPixmap(QPixmap(':/icons/close.png').copy(48, 0, 16, 16))
        self.lockBt.setCheckable(True)
        
        self.horiLayout.addWidget(self.handler)
        self.horiLayout.addWidget(self.lockBt)
        self.horiLayout.addWidget(self.hideBt)
    
        self.lockBt.clicked.connect(self.lockLyric)
        self.hideBt.clicked.connect(self.hideLyric)
        self.timer.timeout.connect(self.hide)
        
    def lockLyric(self):
        if self.lockBt.isChecked():
            self.lockBt.loadPixmap(QPixmap(':/icons/lock.png'))
            self.lockBt.setToolTip('Locked')
            self.lockBt.update()
        else:
            self.lockBt.loadPixmap(QPixmap(':/icons/unlock.png'))
            self.lockBt.setToolTip('Unlocked')
            self.lockBt.update()
    
    def hideLyric(self):
        # self.parent().emit(SIGNAL('lyrichide()'))
        self.parent().lyrichide.emit()
        self.parent().hide()
        self.hide()
                
    def isInTitle(self, xPos, yPos):
        if self.lockBt.isChecked():
            return False
        else:
            return yPos <= self.height() and 0 <= xPos <= self.handler.width()

    def moveEvent(self, event):
        # self.emit(SIGNAL("lyricmoved(QPoint)"), event.pos() - event.oldPos())
        self.lyricmoved.emit(QPoint(event.pos() - event.oldPos()))

    def enterEvent(self, event):
        # print(1)
        self.setFocus()
        self.timer.stop()

    def leaveEvent(self, event):
        self.timer.stop()
        self.timer.start(3000)


def getLyric(rawLyric):
    # copu from https://github.com/wn0112/PPlayer
    lrc = rawLyric

    r1 = re.compile("\[(\d{2}:\d{2}(.\d+)?)\]")
    r2 = re.compile("\[\d+:+.+\](.*)")
    r3 = re.compile("\[offset:(-?\d+)\]")
    item = []
    lrc_lst = []
    offset = 0
    for line in lrc:
        times = r1.findall(line)
        lrc_words = r2.findall(line)
        if lrc_words:
            lrc_words = lrc_words[0]
        else:
            lrc_words = []

        if len(lrc_words) and lrc_words[0].rstrip():
            for i in times:
                item.append(i[0])
                item.append(lrc_words)
                lrc_lst.append(item)
                item = []
    lrc_lst.sort()

    return lrc_lst


if __name__ == '__main__':
    import sys
    os.chdir('..')
    app = QApplication(sys.argv)

    main = PlayWidgets()

    main.show()

    sys.exit(app.exec_())
    


__author__ = 'cyrbuzz'

from singsFrameBase import SingsFrameBase, SingsSearchResultFrameBase


class QQSingsArea(SingsFrameBase):

    def __init__(self, parent=None):
        super(QQSingsArea, self).__init__(parent)


class QQSearchResultFrame(SingsSearchResultFrameBase):

    def __init__(self, parent=None):
        super(QQSearchResultFrame, self).__init__(parent)


import logging
import logging.config

logName = 'running_log.log'

dictConfig = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {
            'format': "%(levelname)s %(module)s  %(lineno)d %(message)s",
        },
    },
    'handlers': {
        # 还没有debug模式。
        'debug': {
            'formatter': 'standard',
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
        'normal': {
            'formatter': 'standard',
            'level': 'INFO',
            'class': 'logging.FileHandler',
            'filename': logName,
            'mode': 'w',
            'encoding': 'utf-8'
        }
    },
    'root': {
            'handlers': ['normal'],
            'level': logging.DEBUG,
            'propagate': True
        }
}

def loggerConfig(logName=None):
    if logName:
        dictConfig['handlers']['normal']['filename'] = logName
        
    logging.config.dictConfig(dictConfig)


"""
重新设计，主要用于熟悉设计模式。
多线程/漂亮界面的设计等。

以网易云音乐为模板。
# 基本没考虑网络的出错问题。
# 
"""

__author__ = 'cyrbuzz'

import os
import sys
import os.path

myFolder = os.path.split(os.path.realpath(__file__))[0]
sys.path = [os.path.join(myFolder, 'widgets'),
os.path.join(myFolder, 'networks'),
os.path.join(myFolder, 'features'),
os.path.join(myFolder, 'apis'),
os.path.join(myFolder, 'logger'),
os.path.join(myFolder, 'dbManager')
] + sys.path

os.chdir(myFolder)

import asyncio
import logging

# event loop
# https://github.com/harvimt/quamash
# an asyncio eventloop for PyQt.
from quamash import QEventLoop

# widgets
from base import (QApplication, cacheFolder, QDialog, QFrame, QHBoxLayout, HBoxLayout, QIcon, QLabel, QListWidget, QListWidgetItem,
                  QPushButton, PicLabel, QScrollArea, ScrollArea, Qt, QTabWidget, TableWidget, QVBoxLayout, VBoxLayout,
                  QWidget)
from player import PlayWidgets
from native import NativeMusic
from downloadFrame import DownloadFrame
from addition import SearchLineEdit
from systemTray import SystemTray
from loginFrames import LoginBox
from singsFrameBase import DetailSings
from netEaseSingsFrames import  NetEaseSingsArea, NetEaseSearchResultFrame
from xiamiSingsFrames import XiamiSingsArea, XiamiSearchResultFrame
from qqSingsFrames import QQSingsArea, QQSearchResultFrame
from recommendFrames import RecommendFrame

# features
from configMainFeatures import (ConfigWindow, ConfigHeader, ConfigNavigation, ConfigMainContent, ConfigSearchArea,
                                ConfigSystemTray, ConfigDetailSings)
from configNativeFeatures import ConfigNative
from configDownloadFrameFeatures import ConfigDownloadFrame
from configNeteaseFeatures import ConfigNetEase
from configXiamiFeatures import ConfigXiami
from configQQFeatures import ConfigQQ
from configRecommendFrameFeatures import ConfigRecommendFrame

# logger
import logger


logger.loggerConfig('logger/running_log.log')

# 覆盖原logger变量。
logger = logging.getLogger(__name__)

logger.info("当前图片缓存目录: {0}".format(os.path.join(os.getcwd(), cacheFolder)))


# 用于承载整个界面。所有窗口的父窗口，所有窗口都可以在父窗口里找到索引。
# 2018/03/18
# 这种嵌套虽然不会出错但有点麻烦，但又不知如何设计得更好。
# 最近在学vue，发现其与Qt有些相似，貌似有个叫vuex的可以很好得管理这些问题。
# 待学习改进。
class Window(QWidget):
    """Window 承载整个界面。"""
    def __init__(self):
        super(Window, self).__init__()
        self.setObjectName('MainWindow')
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setWindowIcon(QIcon('resource/format.ico'))
        self.setWindowTitle("Music")

        self.resize(1022, 670)

        self.header = Header(self)
        self.navigation = Navigation(self)
        self.playWidgets = PlayWidgets(self)
        self.detailSings = DetailSings(self)
        self.mainContent = MainContent(self)
        self.nativeMusic = NativeMusic(self)
        self.downloadFrame = DownloadFrame(self)
        self.searchArea = SearchArea(self)
        self.recommendFrame = RecommendFrame(self)

        self.mainContents = QTabWidget()
        self.mainContents.tabBar().setObjectName("mainTab")

        self.systemTray = SystemTray('resource/logo.png', self)

        # 加载tab设置。
        self.setContents()
        # 添加各类网站的歌单。
        self.addAllPlaylist()
        # 设置布局小细线。
        self.setLines()
        # 设置布局。
        self.setLayouts()
        # 注册功能。
        self.configFeatures()

        with open('QSS/window.qss', 'r') as f:
            self.setStyleSheet(f.read())

    def addAllPlaylist(self):
        self.indexNetEaseSings = NetEaseSingsArea(self.mainContent)
        self.indexXiamiSings = XiamiSingsArea(self.mainContent)
        self.indexQQSings = QQSingsArea(self.mainContent)
        self.mainContent.addTab(self.indexNetEaseSings, "网易云歌单")
        self.mainContent.addTab(self.indexXiamiSings, "虾米歌单")
        self.mainContent.addTab(self.indexQQSings, "QQ歌单")

    # 布局。
    def setContents(self):
        """设置tab界面。"""
        # 将需要切换的窗口做成Tab，并隐藏tabBar，这样方便切换，并且可以做前进后退功能。
        
        self.mainContents.addTab(self.mainContent, '')
        self.mainContents.addTab(self.detailSings, '')
        self.mainContents.addTab(self.nativeMusic, '')
        self.mainContents.addTab(self.downloadFrame, '')
        self.mainContents.addTab(self.recommendFrame, '')
        self.mainContents.addTab(self.searchArea, '')

        self.mainContents.setCurrentIndex(0)

    def setLines(self):
        """设置布局小细线。"""
        self.line1 = QFrame(self)
        self.line1.setObjectName("line1")
        self.line1.setFrameShape(QFrame.HLine)
        self.line1.setFrameShadow(QFrame.Plain)
        self.line1.setLineWidth(2)

    def setLayouts(self):

        self.mainLayout = QVBoxLayout()
        self.mainLayout.addWidget(self.header)
        self.mainLayout.addWidget(self.line1)
        
        self.contentLayout = QHBoxLayout()
        self.contentLayout.setStretch(0, 70)
        self.contentLayout.setStretch(1, 570)
        
        self.contentLayout.addWidget(self.navigation)
        self.contentLayout.addWidget(self.mainContents)

        self.contentLayout.setSpacing(0)
        self.contentLayout.setContentsMargins(0, 0, 0, 0)   

        self.mainLayout.addLayout(self.contentLayout)
        self.mainLayout.addWidget(self.playWidgets)
        
        self.mainLayout.setStretch(0, 43)
        self.mainLayout.setStretch(1, 0)
        self.mainLayout.setStretch(2, 576)
        self.mainLayout.setStretch(3, 50)

        self.mainLayout.setSpacing(0)
        self.mainLayout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.mainLayout)

    # 注册所有功能。
    def configFeatures(self):
        self.config = ConfigWindow(self)
        self.header.config = ConfigHeader(self.header)
        self.downloadFrame.config = ConfigDownloadFrame(self.downloadFrame)
        self.searchArea.config = ConfigSearchArea(self.searchArea)
        self.navigation.config = ConfigNavigation(self.navigation)
        self.nativeMusic.config = ConfigNative(self.nativeMusic)
        self.mainContent.config = ConfigMainContent(self.mainContent)
        self.detailSings.config = ConfigDetailSings(self.detailSings)
        self.indexNetEaseSings.config = ConfigNetEase(self.indexNetEaseSings)
        self.indexXiamiSings.config = ConfigXiami(self.indexXiamiSings)
        self.indexQQSings.config = ConfigQQ(self.indexQQSings)
        self.systemTray.config = ConfigSystemTray(self.systemTray)
        self.recommendFrame.config = ConfigRecommendFrame(self.recommendFrame)

        self.indexNetEaseSings.config.initThread()
        self.indexXiamiSings.config.initThread()
        self.indexQQSings.config.initThread()

        # 当前耦合度过高。
        self.downloadFrame.config.getDownloadSignal()
        
        self.config.pullRecommendSong()

        # move to center.
        screen = QApplication.desktop().availableGeometry()
        self.playWidgets.desktopLyric.resize(screen.width(), 50)
        self.playWidgets.desktopLyric.move(0, screen.height() - 100)

    def closeEvent(self, event):
        # 主要是保存cookies.
        self.header.config.saveCookies()
        self.playWidgets.saveCookies()
        self.downloadFrame.config.saveCookies()

        # 关闭并保存数据库
        self.db.commitAndClose()

        # 系统托盘需要先隐藏，否则退出后会残留在任务栏。
        self.systemTray.hide()

    def resizeEvent(self, event):
       self. playWidgets.currentMusic.move(0, self.height()-64-self.playWidgets.height())


# 标题栏，包括logo，搜索，登陆，最小化/关闭。
class Header(QFrame):

    def __init__(self, parent=None):
        """头部区域，包括图标/搜索/设置/登陆/最大/小化/关闭。"""

        super(Header, self).__init__()
        self.setObjectName('Header')

        self.parent = parent

        self.loginBox = LoginBox(self)

        with open('QSS/header.qss', 'r', encoding='utf-8') as f:
            self.setStyleSheet(f.read())

        # 加载按钮设置。
        self.setButtons()
        # 加载标签设置。
        self.setLabels()
        # 加载输入框设置。
        self.setLineEdits()
        # 加载小细线装饰。
        self.setLines()
        # 加载布局设置。
        self.setLayouts()

    # 布局。
    def setButtons(self):
        """创建所有的按钮。"""

        self.closeButton = QPushButton('×', self)
        self.closeButton.setObjectName("closeButton")
        self.closeButton.setMinimumSize(21, 17)

        self.showminButton = QPushButton('_', self)
        self.showminButton.setObjectName("minButton")
        self.showminButton.setMinimumSize(21, 17)

        self.showmaxButton = QPushButton('□')
        self.showmaxButton.setObjectName("maxButton")
        self.showmaxButton.setMaximumSize(16, 16)

        self.loginButton = QPushButton("未登录 ▼", self)
        self.loginButton.setObjectName("loginButton")

        self.prevButton = QPushButton("<")
        self.prevButton.setObjectName("prevButton")
        self.prevButton.setMaximumSize(28, 22)
        self.prevButton.setMinimumSize(28, 22)

        self.nextButton = QPushButton(">")
        self.nextButton.setObjectName("nextButton")
        self.nextButton.setMaximumSize(28, 22)
        self.nextButton.setMinimumSize(28, 22)

    def setLabels(self):
        """创建所需的所有标签。"""
        self.logoLabel = PicLabel(r'resource/format.png', 32, 32)

        self.descriptionLabel = QLabel(self)
        self.descriptionLabel.setText("<b>Music</b>")

        self.userPix = PicLabel(r'resource/no_music.png', 32, 32, r'resource/user_pic_mask.png')
        self.userPix.setMinimumSize(22, 22)
        self.userPix.setObjectName("userPix")

    def setLineEdits(self):
        """创建搜素框。"""
        self.searchLine = SearchLineEdit(self)
        self.searchLine.setPlaceholderText("搜索音乐, 歌手, 歌词, 用户")

    def setLines(self):
        """设置装饰用小细线。"""
        self.line1 = QFrame(self)
        self.line1.setObjectName("line1")
        self.line1.setFrameShape(QFrame.VLine)
        self.line1.setFrameShadow(QFrame.Plain)
        self.line1.setMaximumSize(1, 25)

    def setLayouts(self):
        """设置布局。"""
        self.mainLayout = QHBoxLayout()
        self.mainLayout.setSpacing(0)
        self.mainLayout.addWidget(self.logoLabel)
        self.mainLayout.addWidget(self.descriptionLabel)
        self.mainLayout.addSpacing(70)
        self.mainLayout.addWidget(self.prevButton)
        self.mainLayout.addWidget(self.nextButton)
        self.mainLayout.addSpacing(10)
        self.mainLayout.addWidget(self.searchLine)
        self.mainLayout.addStretch(1)
        self.mainLayout.addWidget(self.userPix)
        self.mainLayout.addSpacing(7)
        self.mainLayout.addWidget(self.loginButton)
        self.mainLayout.addSpacing(7)
        self.mainLayout.addWidget(self.line1)
        self.mainLayout.addSpacing(30)
        self.mainLayout.addWidget(self.showminButton)
        self.mainLayout.addWidget(self.showmaxButton)
        self.mainLayout.addSpacing(3)
        self.mainLayout.addWidget(self.closeButton)

        self.setLayout(self.mainLayout)

    # 事件。
    """重写鼠标事件，实现窗口拖动。"""
    def mousePressEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            self.parent.m_drag = True
            self.parent.m_DragPosition = event.globalPos()-self.parent.pos()
            event.accept()

    def mouseMoveEvent(self, event):
        try:
            if event.buttons() and Qt.LeftButton:
                self.parent.move(event.globalPos()-self.parent.m_DragPosition)
                event.accept()
        except AttributeError:
            pass

    def mouseReleaseEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            self.m_drag = False


# 左侧的导航栏，包括发现音乐/歌单/本地音乐。
class Navigation(QScrollArea):
    def __init__(self, parent=None):
        """包括发现音乐，MV，我的音乐, 歌单等导航信息。"""
        super(Navigation, self).__init__(parent)
        self.parent = parent
        self.frame = QFrame()
        self.setMaximumWidth(200)

        self.setWidget(self.frame)
        self.setWidgetResizable(True)
        self.frame.setMinimumWidth(200)

        # 定义3个事件函数，方便扩展。
        self.navigationListFunction = self.none
        self.nativeListFunction = self.none
        self.singsFunction = self.none

        with open('QSS/navigation.qss', 'r') as f:
            style = f.read()
            self.setStyleSheet(style)
            self.frame.setStyleSheet(style)

        # 包括显示信息： 推荐 我的音乐 歌单。
        self.setLabels()
        # 包括详细的内容：发现音乐，FM，MV等。
        self.setListViews()

        self.setLayouts()

    # 布局。
    def setLabels(self):
        """定义所有的标签。"""
        self.recommendLabel = QLabel(" 推荐")
        self.recommendLabel.setObjectName("recommendLabel")
        self.recommendLabel.setMaximumHeight(27)

        self.myMusic = QLabel(" 我的音乐")
        self.myMusic.setObjectName("myMusic")
        self.myMusic.setMaximumHeight(27)
        # self.myMusic.setMaximumHeight(54)

        self.singsListLabel = QLabel(" 收藏与创建的歌单")
        self.singsListLabel.setObjectName("singsListLabel")
        self.singsListLabel.setMaximumHeight(27)

    def setListViews(self):
        """定义承载功能的ListView"""
        self.navigationList = QListWidget()
        self.navigationList.setMaximumHeight(110)
        self.navigationList.setObjectName("navigationList")
        self.navigationList.addItem(QListWidgetItem(QIcon('resource/music.png'), " 发现音乐"))
        self.navigationList.addItem(QListWidgetItem(QIcon('resource/signal.png'), " 私人FM"))
        self.navigationList.addItem(QListWidgetItem(QIcon('resource/movie.png'), " MV"))
        self.navigationList.setCurrentRow(0)

        self.nativeList = QListWidget()
        self.nativeList.setObjectName("nativeList")
        self.nativeList.setMaximumHeight(100)
        self.nativeList.addItem(QListWidgetItem(QIcon('resource/notes.png')," 本地音乐"))
        self.nativeList.addItem(QListWidgetItem(QIcon('resource/download_icon.png'), " 我的下载"))
        self.nativeList.addItem(QListWidgetItem(QIcon('resource/recommend_icon.png'), " 专属推荐"))

    def setLayouts(self):
        """定义布局。"""
        self.mainLayout = VBoxLayout(self.frame)
        self.mainLayout.addSpacing(10)
        self.mainLayout.addWidget(self.recommendLabel)
        self.mainLayout.addSpacing(3)
        self.mainLayout.addWidget(self.navigationList)
        self.mainLayout.addSpacing(1)
        
        self.mainLayout.addWidget(self.myMusic)
        self.mainLayout.addSpacing(3)
        self.mainLayout.addWidget(self.nativeList)
        self.mainLayout.addSpacing(1)

        self.mainLayout.addWidget(self.singsListLabel)
        self.mainLayout.addSpacing(1)

        self.mainLayout.addStretch(1)

        self.setContentsMargins(0, 0, 0, 0)

    # just a test.
    def setSingsList(self):

        pass

    # 功能。
    def none(self):
        # 没有用的空函数。
        pass


# 主要内容区，包括最新的歌单。
class MainContent(ScrollArea):
    # 定义一个滑到了最低部的信号。
    # 方便子控件得知已经滑到了最底部，要做些加载的动作。

    def __init__(self, parent=None):
        """主内容区，包括推荐歌单等。"""
        super(MainContent, self).__init__()
        self.parent = parent
        self.setObjectName("MainContent")

        # 连接导航栏的按钮。
        # self.parent.navigation.navigationListFunction = self.navigationListFunction
        with open("QSS/mainContent.qss", 'r', encoding='utf-8') as f:
            self.style = f.read()
            self.setStyleSheet(self.style)

        self.tab = QTabWidget()
        self.tab.setObjectName("contentsTab")

        self.mainLayout = QVBoxLayout()
        self.mainLayout.setSpacing(0)
        self.mainLayout.setContentsMargins(0, 0, 0, 0)
        self.mainLayout.addWidget(self.tab)

        self.frame.setLayout(self.mainLayout)

    def addTab(self, widget, name=''):
        self.tab.addTab(widget, name)


# 搜索后的结果显示页。
class SearchArea(ScrollArea):

    def __init__(self, parent=None):
        super(SearchArea, self).__init__(self)
        self.parent = parent
        self.setObjectName("searchArea")
        with open('QSS/searchArea.qss', 'r', encoding='utf-8') as f:
            self.setStyleSheet(f.read())

        self.mainLayout = QVBoxLayout(self.frame)

        self.titleLabel = QLabel(self.frame)

        # 搜索结果的tab。
        self.contentsTab = QTabWidget(self.frame)

        # 加入布局。
        self.mainLayout.addWidget(self.titleLabel)
        self.mainLayout.addWidget(self.contentsTab)

        self.setSingsFrame()

    # 布局。
    def setSingsFrame(self):
        # 单曲界面。
        self.neteaseSearchFrame = NetEaseSearchResultFrame(self)
        self.contentsTab.addTab(self.neteaseSearchFrame, "网易云")

        self.xiamiSearchFrame = XiamiSearchResultFrame(self)
        self.contentsTab.addTab(self.xiamiSearchFrame, "虾米")

        self.qqSearchFrame = QQSearchResultFrame(self)
        self.contentsTab.addTab(self.qqSearchFrame, 'QQ')

    # 功能。
    def setText(self, text):
        self.text = text
        self.titleLabel.setText("搜索<font color='#23518F'>“{0}”</font><br>".format(self.text))


def start():
    app = QApplication(sys.argv)

    # 将Qt事件循环写到asyncio事件循环里。
    # QEventLoop不是Qt原生事件循环，
    # 是被asyncio重写的事件循环。
    eventLoop = QEventLoop(app)
    asyncio.set_event_loop(eventLoop)

    try:
        main = Window()

        main.show()
        # 当前音乐的显示信息。
        # 因为需要布局之后重新绘制的宽高。
        # 这个宽高会在show之后才会改变。
        # 需要获取宽，高并嵌入到父窗口里。
        main.playWidgets.currentMusic.resize(main.navigation.width(), 64)
        
        with eventLoop:
            eventLoop.run_forever()

        sys.exit(0)
    except:
        logger.error("got some error", exc_info=True)


if __name__ == '__main__':
    start()    
